--------------------------------------------------------
-- CREACIO DE LES TAULES PRINCIPALS DE LA PLATAFORMA  --
--------------------------------------------------------

---------------------
-- Taula de Paisos --
---------------------
CREATE TABLE PAIS (
   CODI CHAR(2 CHAR) CONSTRAINT PK_PAIS PRIMARY KEY, 
	 NOM VARCHAR2(50 CHAR) CONSTRAINT NN_PAIS_NOM NOT NULL
);
  
--------------------------------
-- Taula Operadors Telefonics --
--------------------------------
CREATE TABLE OPERADOR (
   CODI CHAR(4 CHAR) CONSTRAINT PK_OPERADOR PRIMARY KEY, 
	 NOM VARCHAR2(50 CHAR) CONSTRAINT NN_OPERADOR_NOM NOT NULL
);

------------------------------
-- Taula Models Dispositius --
------------------------------
CREATE TABLE MODELDISPOSITIU (
   CODI CHAR(7 CHAR) CONSTRAINT PK_MODELDISPOSITIU PRIMARY KEY, 
	 NOM VARCHAR2(50 CHAR) CONSTRAINT NN_MODELDISPOSITIU_NOM NOT NULL
);

------------------------------
-- Taula Sistemes Operatius --
------------------------------
CREATE TABLE SO (
   CODI CHAR(7 CHAR) CONSTRAINT PK_SO PRIMARY KEY, 
	 NOM VARCHAR2(50 CHAR) CONSTRAINT NN_SO_NOM NOT NULL
);

---------------------
-- Taula d'idiomes --
---------------------
CREATE TABLE IDIOMA (
   CODI NUMBER(2) CONSTRAINT PK_IDIOMA PRIMARY KEY, 
	 NOM VARCHAR2(30 CHAR) CONSTRAINT NN_IDIOMA_NOM NOT NULL
);

--------------------------
-- Taula de Resolucions --
--------------------------
CREATE TABLE RESOLUCIO (
   CODI NUMBER(2) CONSTRAINT PK_RESOLUCIO PRIMARY KEY, 
	 NOM VARCHAR2(10 CHAR) CONSTRAINT NN_RESOLUCIO_NOM NOT NULL
);

--------------------------------
-- Taula de Modes de Pagament --
--------------------------------
CREATE TABLE PAGAMENT (
   CODI NUMBER(2) CONSTRAINT PK_PAGAMENT PRIMARY KEY, 
	 NOM VARCHAR2(30 CHAR) CONSTRAINT NN_PAGAMENT_NOM NOT NULL
);

-------------------------
-- Taula d'Aplicacions --
-------------------------
CREATE TABLE APLICACIO (
   CODI NUMBER CONSTRAINT PK_APLICACIO PRIMARY KEY, 
	 IDAPLICACIO VARCHAR2(50 CHAR) CONSTRAINT NN_APLICACIO_IDAPLICACIO NOT NULL,
   DATAPUJADA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP,
   DATABAIXA TIMESTAMP WITH LOCAL TIME ZONE,
   VERSIO VARCHAR2 (10 CHAR),
   LINKVIDEO VARCHAR2 (50 CHAR),
   RESOLUCIOMINIMA NUMBER(2) CONSTRAINT FK1_APLICACIO_RESOLUCIO REFERENCES RESOLUCIO(CODI),
   ACTIVA CHAR(1 CHAR) DEFAULT 'N'
                      CONSTRAINT CH_APLICACIO_ACTIVA CHECK (ACTIVA IN ('S', 'N'))
);

-------------------------------------------
-- Taula Vincles Aplicacio amb Preu/Pais --
-------------------------------------------
CREATE TABLE APLICACIOPREU (
  CODIAPLICACIO NUMBER CONSTRAINT FK1_APLICACIOPREU_APLICACIO REFERENCES APLICACIO(CODI),  
  CODIPAIS CHAR(2 CHAR) CONSTRAINT FK2_APLICACIOPREU_PAIS REFERENCES PAIS(CODI),
  PREU NUMBER(4,2) CONSTRAINT NN_APLICACIOPREU_PREU NOT NULL,
  CONSTRAINT PK_APLICACIOPREU PRIMARY KEY (CODIAPLICACIO,CODIPAIS)
);

----------------------------------------------------
-- Taula Vincles Aplicacio amb Sistemes Operatius --
----------------------------------------------------
CREATE TABLE APLICACIOSO (
  CODIAPLICACIO NUMBER CONSTRAINT FK1_APLICACIOSO_APLICACIO REFERENCES APLICACIO(CODI),  
  CODISO CHAR(7 CHAR) CONSTRAINT FK2_APLICACIOSO_SO REFERENCES SO(CODI),
  CONSTRAINT PK_APLICACIOSO PRIMARY KEY (CODIAPLICACIO,CODISO)
);

----------------------------------------------
-- Taula Vincles Aplicacio amb Descripcions --
----------------------------------------------
CREATE TABLE APLICACIODESCRIPCIO (
  CODIAPLICACIO NUMBER CONSTRAINT FK1_APLICACIODESC_APLICACIO REFERENCES APLICACIO(CODI),  
  CODIIDIOMA NUMBER(2) CONSTRAINT FK2_APLICACIODESC_IDIOMA REFERENCES IDIOMA(CODI),  
  DESCRIPCIO VARCHAR2 (100 CHAR) CONSTRAINT NN_APLICACIODESCRIPCIO NOT NULL,
  CONSTRAINT PK_APLICACIODESCRIPCIO PRIMARY KEY (CODIAPLICACIO,CODIIDIOMA)
);

-------------------------------------------------
-- Taula de Fitxers Binaris --
-------------------------------------------------
CREATE TABLE BINARI (
  CODI NUMBER CONSTRAINT PK_BINARI PRIMARY KEY,
  CODIAPLICACIO NUMBER CONSTRAINT FK1_BINARI_APLICACIO REFERENCES APLICACIO(CODI)
                      CONSTRAINT NN_BINARI_CODIAPLICACIO NOT NULL,  
  CODISO CHAR(7 CHAR) CONSTRAINT FK2_BINARI_SO REFERENCES SO(CODI)
                      CONSTRAINT NN_BINARI_CODISO NOT NULL,
  VERSIO VARCHAR2 (10 CHAR) CONSTRAINT NN_BINARI_VERSIO NOT NULL,
  DATAPUJADA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP,
  DATABAIXA TIMESTAMP WITH LOCAL TIME ZONE,
  HIPERVINCLE VARCHAR2 (50 CHAR) CONSTRAINT NN_BINARI_HIPERVINCLE NOT NULL,
  MIDA NUMBER CONSTRAINT NN_BINARI_MIDA NOT NULL, /*en bytes*/
  RESOLUCIO NUMBER(2) CONSTRAINT FK3_APLICACIOBIN_RESOLUCIO REFERENCES RESOLUCIO(CODI),
  ACTIVA CHAR(1 CHAR) DEFAULT 'N' 
                      CONSTRAINT CH_APLICACIOBIN_ACTIVA CHECK (ACTIVA IN ('S', 'N'))
);

------------------------------
-- Taula de Desenvolupadors --
------------------------------
CREATE TABLE DESENVOLUPADOR (
   CODI NUMBER CONSTRAINT PK_DESENVOLUPADOR PRIMARY KEY, 
	 IDDESENVOLUPADOR VARCHAR2(50 CHAR) CONSTRAINT NN_DESENVOLUPADOR_ID NOT NULL,
   DATAALTA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP,
   DATABAIXA TIMESTAMP WITH LOCAL TIME ZONE,
   NOMEMPRESA VARCHAR2 (50 CHAR),
   NOMREPLEGAL VARCHAR2 (50 CHAR),
   ADREA VARCHAR2 (200 CHAR),
   TELEFON VARCHAR2 (20 CHAR),
   PAIS CHAR(2 CHAR) CONSTRAINT FK1_DESENVOLUPADOR_PAIS REFERENCES PAIS(CODI)
                    CONSTRAINT NN_DESENVOLUPADOR_PAIS NOT NULL
);

------------------------------------------------------
-- Taula Vincles Desenvolupador amb Fitxers Binaris --
------------------------------------------------------
CREATE TABLE DESENVOLUPADORBINARI (
  CODIDESENVOLUPADOR NUMBER CONSTRAINT FK1_DESENVOLUPADORBIN_DES REFERENCES DESENVOLUPADOR(CODI), 
  CODIBINARI NUMBER CONSTRAINT FK2_DESENVOLUPADORBIN_BIN REFERENCES BINARI(CODI),
  CONSTRAINT PK_DESENVOLUPADORBIN PRIMARY KEY (CODIDESENVOLUPADOR,CODIBINARI)
);

---------------------
-- Taula d'Usuaris --
---------------------
CREATE TABLE USUARI (
   CODI NUMBER CONSTRAINT PK_USUARI PRIMARY KEY, 
   DATAALTA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP,
   DATABAIXA TIMESTAMP WITH LOCAL TIME ZONE,
   MOBIL VARCHAR2 (20 CHAR) CONSTRAINT NN_USUARI_MOBIL NOT NULL,
   OPERADOR CHAR(4 CHAR) CONSTRAINT FK1_USUARI_OPERADOR REFERENCES OPERADOR(CODI)
                        CONSTRAINT NN_USUARI_OPERADOR NOT NULL,
   EMAIL VARCHAR2 (50 CHAR),
   PAIS CHAR(2 CHAR) CONSTRAINT FK2_USUARI_PAIS REFERENCES PAIS(CODI)
                    CONSTRAINT NN_USUARI_PAIS NOT NULL
);

--------------------------
-- Taula de Dispositius --
--------------------------
CREATE TABLE DISPOSITIU (
   CODI NUMBER CONSTRAINT PK_DISPOSITIU PRIMARY KEY, 
   IMEI VARCHAR2 (30 CHAR),
   MODELDISP CHAR(7 CHAR) CONSTRAINT FK1_DISPOSITIU_MODEL REFERENCES MODELDISPOSITIU(CODI)
                          CONSTRAINT NN_DISPOSITIU_MODELDISP NOT NULL,
   RESOLUCIO NUMBER(2) CONSTRAINT FK2_DISPOSITIU_RESOLUCIO REFERENCES RESOLUCIO(CODI),
   USUARI NUMBER CONSTRAINT FK3_DISPOSITIU_USUARI REFERENCES USUARI(CODI)
                CONSTRAINT NN_DISPOSITIU_USUARI NOT NULL,
   SO CHAR(7 CHAR) CONSTRAINT FK4_DISPOSITIU_SO REFERENCES SO(CODI)
                CONSTRAINT NN_DISPOSITIU_SO NOT NULL                
);

---------------------------
-- Taula de Descarregues --
---------------------------
CREATE TABLE DESCARREGA (
  CODIDISPOSITIU NUMBER CONSTRAINT FK1_DESCARREGA_DISPOSITIU REFERENCES DISPOSITIU(CODI)
                CONSTRAINT NN_DESCARREGA_DISPOSITIU NOT NULL,
  CODIBINARI NUMBER CONSTRAINT FK2_DESCARREGA_BINARI REFERENCES BINARI(CODI)
                CONSTRAINT NN_DESCARREGA_BINARI NOT NULL,
  CODIUSUARI NUMBER CONSTRAINT FK3_DESCARREGA_USUARI REFERENCES USUARI(CODI)
                CONSTRAINT NN_DESCARREGA_USUARI NOT NULL,
  CODIAPLICACIO NUMBER CONSTRAINT FK4_DESCARREGA_APLICACIO REFERENCES APLICACIO(CODI)
                CONSTRAINT NN_DESCARREGA_APLICACIO NOT NULL,
  DATAHORA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP,
  MODEPAGAMENT NUMBER(2) CONSTRAINT FK5_DESCARREGA_PAG REFERENCES PAGAMENT(CODI)
                      CONSTRAINT NN_DESCARREGA_MODEPAGAMENT NOT NULL,
  OPERADOR CHAR(4 CHAR) CONSTRAINT FK6_DESCARREGA_OPERADOR REFERENCES OPERADOR(CODI)
                        CONSTRAINT NN_DESCARREGA_OPERADOR NOT NULL,
  PREU NUMBER(4,2) CONSTRAINT NN_DESCARREGA_PREU NOT NULL,
  PAIS CHAR(2 CHAR) CONSTRAINT FK7_DESCARREGA_PAIS REFERENCES PAIS(CODI)
                    CONSTRAINT NN_DESCARREGA_PAIS NOT NULL,
  CONSTRAINT PK_DESCARREGA PRIMARY KEY (CODIDISPOSITIU,CODIBINARI,DATAHORA)
);

------------------
-- Taula de LOG --
------------------
CREATE TABLE TLOG (
  DATAHORA TIMESTAMP WITH LOCAL TIME ZONE DEFAULT SYSTIMESTAMP
            CONSTRAINT NN_LOG_DATAHORA NOT NULL,
  PROCEDIMENT VARCHAR2 (50 CHAR) CONSTRAINT NN_LOG_PROCEDIMENT NOT NULL,
  PARINPUT VARCHAR2 (4000 CHAR),
  PAROUTPUT VARCHAR2 (4000 CHAR),
  RESULTAT VARCHAR2 (300 CHAR)
);
