----------------------------------------------------
-- CREACIO DELS DISPARADORS DEL MODUL ESTADISTIC  --
----------------------------------------------------
/* Si no es canvia el valor del seguent parametre de sessi es produeixen errors de compilaci dels triggers des de SQL Developer */
ALTER SESSION SET PLSCOPE_SETTINGS = 'IDENTIFIERS:NONE';

-------------------------------------------------------------------------------------------------------
-- Disparador per a que la mitjana de descarregues anuals de la taula ESTADISTIQUESANUALS            --
-- s'actualitzi automaticament quan el nmero de descarregues o els usuaris s'han afegit o modificat --
-------------------------------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER trgMITJANADESCARREGUES
BEFORE INSERT OR UPDATE OF DESCARREGUES, USUARIS ON ESTADISTIQUESANUALS FOR EACH ROW 
BEGIN
    IF (:new.DESCARREGUES IS NOT NULL) AND (:new.USUARIS IS NOT NULL) THEN 
        IF (:new.DESCARREGUES = 0) OR (:new.USUARIS =0) THEN
          :new.MITJANADESCARREGUES := 0;
        ELSE
          :new.MITJANADESCARREGUES := (:new.DESCARREGUES / :new.USUARIS) ;
        END IF;
    ELSE
        :new.MITJANADESCARREGUES := 0;
    END IF; 
END;
/

---------------------------------------------------------------------
-- Disparador per a que s'actualitzi el desenvolupador amb         --
-- el mxim nombre de descrregues de la taula ESTADISTIQUESANUALS --
---------------------------------------------------------------------
CREATE OR REPLACE TRIGGER trgMAXDESENVOLUPADOR FOR INSERT OR UPDATE ON ESTADISTIQUESANUALSDES COMPOUND TRIGGER -- Compound trigger per a evitar taules mutants (una select sobre la mateixa taula del trigger)
-- emmagatzema el nombre de descarregues mxim
intMaxDescarregues NUMBER; 
-- emmagatzema els desenvolupadors que han assolit el nombre mxim de descarregues
txtDesenvolupadors VARCHAR2 (500 CHAR); 
-- emmagatzema l'any per a poder-lo usar posteriorment
txtAny VARCHAR2 (4 CHAR); 
-- obtenim els desenvolupadors amb maxim nombre de descarregues
CURSOR curDESENVOLUPADOR IS SELECT DESENVOLUPADOR FROM ESTADISTIQUESANUALSDES WHERE (ANYESTADISTICA = txtAny) AND (DESCARREGUES = intMaxDescarregues); 

AFTER EACH ROW IS
BEGIN
  txtAny := :new.ANYESTADISTICA;
END AFTER EACH ROW;

AFTER STATEMENT IS
BEGIN
  txtDesenvolupadors := '';
  -- seleccionem el valor mxim de descarregues
  SELECT MAX(DESCARREGUES) INTO intMaxDescarregues FROM ESTADISTIQUESANUALSDES WHERE ANYESTADISTICA = txtAny; 
  
  FOR desenv IN curDESENVOLUPADOR
  LOOP
    IF txtDesenvolupadors IS NOT NULL THEN 
      txtDesenvolupadors := txtDesenvolupadors || ';';
    END IF;   
    txtDesenvolupadors := txtDesenvolupadors || desenv.DESENVOLUPADOR;   
  END LOOP;
  UPDATE ESTADISTIQUESANUALS SET DESENVOLUPADORMAXDESC = txtDesenvolupadors, DESENVOLUPADORMAXDESCNUMERO = intMaxDescarregues WHERE ANYESTADISTICA = txtAny ;

END AFTER STATEMENT;  
END;
/

-------------------------------------------------------
-- Disparador per a que s'actualitzi l'aplicaci amb --
-- ms racaptaci de la taula ESTADISTIQUESANUALS    --
-------------------------------------------------------
-- Compound trigger per a evitar taules mutants (una select sobre la mateixa taula del trigger)
CREATE OR REPLACE TRIGGER trgMAXAPLICACIO FOR INSERT OR UPDATE ON ESTADISTIQUESANUALSAPP COMPOUND TRIGGER 
-- emmagatzema la mxima recaptaci
intMaxRecaptacio NUMBER(7,2); 
-- emmagatzema les aplicacions que han assolit la mxima recaptaci
txtAplicacions VARCHAR2 (500 CHAR); 
-- emmagatzema els desenvolupadors que han assolit la mxima recaptaci
txtDesenvolupadors VARCHAR2 (500 CHAR); 
-- emmagatzema l'any per a poder-lo usar posteriorment
txtAny CHAR (4 CHAR); 
-- obtenim l'aplicacio i els desenvolupadors amb la mxima recaptacio
CURSOR curAPLICACIO IS SELECT APLICACIO,DESENVOLUPADORS FROM ESTADISTIQUESANUALSAPP WHERE (ANYESTADISTICA = txtAny) AND (EUROSDESCARREGUES = intMaxRecaptacio); 

AFTER EACH ROW IS
BEGIN
  txtAny := :new.ANYESTADISTICA;
END AFTER EACH ROW;

AFTER STATEMENT IS
BEGIN
  txtDesenvolupadors := '';
  txtAplicacions := '';
  -- seleccionem el valor mxim de recaptacio
  SELECT MAX(EUROSDESCARREGUES) INTO intMaxRecaptacio FROM ESTADISTIQUESANUALSAPP WHERE ANYESTADISTICA = txtAny; 
  
  FOR dadesAplicacio IN curAPLICACIO
  LOOP
    IF txtAplicacions IS NOT NULL THEN 
      txtAplicacions := txtAplicacions || '|';
    END IF;   
    txtAplicacions := txtAplicacions || dadesAplicacio.APLICACIO;    
    IF txtDesenvolupadors IS NOT NULL THEN 
      txtDesenvolupadors := txtDesenvolupadors || '|';
    END IF;   
    txtDesenvolupadors := txtDesenvolupadors || dadesAplicacio.DESENVOLUPADORS;   
  END LOOP;
  
  UPDATE ESTADISTIQUESANUALS SET APLICACIOMAXRECAPTACIO = txtAplicacions, APLICACIOMAXRECAPTACIODES = txtDesenvolupadors WHERE ANYESTADISTICA = txtAny ;

END AFTER STATEMENT;  
END;
/

-----------------------------------------------------------------------------------------
-- Disparador per actualitzar les taules del mdul estadistic.                         --
-- S'activa quan es fa una descrrega a la plataforma (inserci a la taula DESCARREGA) --
-----------------------------------------------------------------------------------------
-- Compound trigger per a evitar taules mutants (una select sobre la mateixa taula del trigger)
CREATE OR REPLACE TRIGGER trgDESCARREGA FOR INSERT ON DESCARREGA COMPOUND TRIGGER 
  intComptador NUMBER := 0;
  intAny CHAR (4 CHAR);
  intUsuaris NUMBER := 0;
  intBinari NUMBER;
  txtAplicacio VARCHAR2(50 CHAR);
  txtDesenvolupadors VARCHAR2(500 CHAR);
  intPreu NUMBER;
  datDataHora DATE;
  txtPais CHAR(2 CHAR);
  intCodiUsuari NUMBER;
  intCodiAplicacio NUMBER;
  -- obtenim els desenvolupadors del fitxer binari descarregar  
  CURSOR curDESENVOLUPADORS IS SELECT IDDESENVOLUPADOR FROM DESENVOLUPADOR WHERE CODI IN (SELECT CODIDESENVOLUPADOR FROM DESENVOLUPADORBINARI WHERE CODIBINARI = intBinari); 

AFTER EACH ROW IS
BEGIN
  intBinari := :new.CODIBINARI;
  intPreu := :new.PREU;
  datDataHora := :new.DATAHORA;
  txtPais := :new.PAIS;
  intCodiUsuari := :new.CODIUSUARI;
  intCodiAplicacio := :new.CODIAPLICACIO;
END AFTER EACH ROW;

AFTER STATEMENT IS
BEGIN
  -- Actualitzaci de la taula ESTADISTIQUESGENERALS (DESCARREGUES,EUROSDESCARREGUES)
  UPDATE ESTADISTIQUESGENERALS SET DESCARREGUES = DESCARREGUES + 1, EUROSDESCARREGUES = EUROSDESCARREGUES + intPreu;
  
  -- Actualitzaci de la taula ESTADISTIQUESANUALS, camp DESCARREGUES
  -- Comprovem que existeixi el registre per l'any corresponent, sino, es crea amb el nombre d'usuaris actius actualment
  intAny := EXTRACT(YEAR FROM datDataHora);
  SELECT COUNT(*) INTO intComptador FROM ESTADISTIQUESANUALS WHERE ANYESTADISTICA = intAny;
  IF (intComptador = 0) THEN
    SELECT COUNT (*) INTO intUsuaris FROM USUARI WHERE DATABAIXA IS NULL;
    INSERT INTO ESTADISTIQUESANUALS (ANYESTADISTICA,USUARIS) VALUES (intAny,intUsuaris);
  END IF;
  
  UPDATE ESTADISTIQUESANUALS SET DESCARREGUES = DESCARREGUES + 1 WHERE ANYESTADISTICA = intAny;
  
  -- Actualitzaci de la taula ESTADISTIQUESANUALSDES, camps DESENVOLUPADOR i DESCARREGUES,  automaticament actualitzar la taula ESTADISTIQUESANUALS (DESENVOLUPADORMAXDESC,DESENVOLUPADORMAXDESCNUMERO)
  -- Busquem els desenvolupadors de l'aplicaci i per cadascun d'ells actualitzem el comptador de descarregues
  -- Aprofitem que fem un bucle per emmagatzemar aquests desenvolupadors, serviran posteriorment per actualitzar la taula ESTADISTIQUESANUALSAPP
  txtDesenvolupadors := '';
  FOR idDesenvolupador IN curDESENVOLUPADORS
  LOOP
    -- Comprovem si existeix el registre per aquell desenvolupador
    SELECT COUNT(*) INTO intComptador FROM ESTADISTIQUESANUALSDES WHERE (ANYESTADISTICA = intAny) AND (DESENVOLUPADOR = idDesenvolupador.IDDESENVOLUPADOR);
    IF (intComptador = 0) THEN
      INSERT INTO ESTADISTIQUESANUALSDES (ANYESTADISTICA,DESENVOLUPADOR,DESCARREGUES) VALUES (intAny,idDesenvolupador.IDDESENVOLUPADOR,1);
    ELSE
      UPDATE ESTADISTIQUESANUALSDES SET DESCARREGUES = DESCARREGUES + 1 WHERE (ANYESTADISTICA = intAny) AND (DESENVOLUPADOR = idDesenvolupador.IDDESENVOLUPADOR);
    END IF;
    -- Construim la cadena amb els desenvolupadors
    IF txtDesenvolupadors IS NOT NULL THEN 
      txtDesenvolupadors := txtDesenvolupadors || ';';
    END IF;   
    txtDesenvolupadors := txtDesenvolupadors || idDesenvolupador.IDDESENVOLUPADOR;    
  END LOOP;

  -- Actualitzaci de la taula ESTADISTIQUESANUALSAPP (automaticament actualitzar la taula ESTADISTIQUESANUALS)
  -- Obtenim l'aplicacio
  SELECT IDAPLICACIO INTO txtAplicacio FROM APLICACIO WHERE CODI IN (SELECT CODIAPLICACIO FROM BINARI WHERE CODI = intBinari); 
  -- Comprovem si existeix el registre per aquesta aplicacio
  SELECT COUNT(*) INTO intComptador FROM ESTADISTIQUESANUALSAPP WHERE (ANYESTADISTICA = intAny) AND (APLICACIO = txtAplicacio);
  IF (intComptador = 0) THEN
    INSERT INTO ESTADISTIQUESANUALSAPP (ANYESTADISTICA,APLICACIO,DESENVOLUPADORS,EUROSDESCARREGUES) VALUES (intAny,txtAplicacio,txtDesenvolupadors,intPreu);
  ELSE
    UPDATE ESTADISTIQUESANUALSAPP SET EUROSDESCARREGUES = EUROSDESCARREGUES + intPreu WHERE (ANYESTADISTICA = intAny) AND (APLICACIO = txtAplicacio);
  END IF;

  -- Actualitzaci de la taula ESTADISTIQUESANUALSPAIS
  -- Primer mirem si existeix el registre, sin el creem
  SELECT COUNT(*) INTO intComptador FROM ESTADISTIQUESANUALSPAIS WHERE (ANYESTADISTICA = intAny) AND (PAIS = txtPais);
  IF (intComptador = 0) THEN
    INSERT INTO ESTADISTIQUESANUALSPAIS (ANYESTADISTICA,PAIS) VALUES (intAny,txtPais);
  END IF;
  -- Comprovem si es un usuari diferent (ha de ser la primera descarrega d'aquest usuari)
  SELECT COUNT(*) INTO intComptador FROM DESCARREGA WHERE (EXTRACT(YEAR FROM DATAHORA) = intAny) AND (CODIUSUARI = intCodiUsuari) AND (PAIS = txtPAIS);
  IF (intComptador = 1) THEN
    UPDATE ESTADISTIQUESANUALSPAIS SET USUARISDIF = USUARISDIF + 1 WHERE (ANYESTADISTICA = intAny) AND (PAIS = txtPais);
  END IF;  
  -- Comprovem si es una aplicaci diferent (ha de ser la primera descarrega d'aquesta aplicaci)
  SELECT COUNT(*) INTO intComptador FROM DESCARREGA WHERE (EXTRACT(YEAR FROM DATAHORA) = intAny) AND (CODIAPLICACIO = intCodiAplicacio) AND (PAIS = txtPAIS);
  IF (intComptador = 1) THEN
    UPDATE ESTADISTIQUESANUALSPAIS SET APLICACIONSDIF = APLICACIONSDIF + 1 WHERE (ANYESTADISTICA = intAny) AND (PAIS = txtPais);
  END IF;    
  -- Finalment actualitzem els ingressos
  UPDATE ESTADISTIQUESANUALSPAIS SET INGRESSOSUSUARIS = INGRESSOSUSUARIS + intPreu WHERE (ANYESTADISTICA = intAny) AND (PAIS = txtPais);
  
END AFTER STATEMENT;  
END;
/

-----------------------------------------------------------------------------------------
-- Disparador per actualitzar la taula ESTADISTIQUESANUALS (USUARIS)                   --
-- S'activa quan es crea un nou usuari a la plataforma (inserci a la taula USUARI)    --
-----------------------------------------------------------------------------------------
-- Compound trigger per a evitar taules mutants (una select sobre la mateixa taula del trigger)
CREATE OR REPLACE TRIGGER trgUSUARINOU FOR INSERT ON USUARI COMPOUND TRIGGER 
-- emmagatzema l'any per a poder-lo usar posteriorment
txtAny CHAR (4 CHAR); 
intComptador NUMBER := 0;
intUsuaris NUMBER := 0;

AFTER EACH ROW IS
BEGIN
  -- Comprovem que existeixi el registre per l'any corresponent, sino, es crea amb el nombre d'usuaris actius actualment
  txtAny := EXTRACT(YEAR FROM :new.DATAALTA);
END AFTER EACH ROW;

AFTER STATEMENT IS
BEGIN
  -- Comprovem que existeixi el registre per l'any corresponent, sino, es crea amb el nombre d'usuaris actius actualment
  SELECT COUNT(*) INTO intComptador FROM ESTADISTIQUESANUALS WHERE ANYESTADISTICA = txtAny;
  IF (intComptador = 0) THEN
    SELECT COUNT (*) INTO intUsuaris FROM USUARI WHERE DATABAIXA IS NULL;
    INSERT INTO ESTADISTIQUESANUALS (ANYESTADISTICA,USUARIS) VALUES (txtAny,intUsuaris);
  ELSE
    UPDATE ESTADISTIQUESANUALS SET USUARIS = USUARIS +1 WHERE ANYESTADISTICA = txtAny;
  END IF;
END AFTER STATEMENT;  
END;
/

/* Es restaura el valor per defecte que SQL Developer posa al seguent parametre de sessi */
ALTER SESSION SET PLSCOPE_SETTINGS = 'IDENTIFIERS:ALL';