----------------------------------------------------------------------------------------
-- CREACIO DE LES FUNCIONS NECESSARIES PER SATISFER LES CONSULTES AL MODUL ESTADISTIC --
----------------------------------------------------------------------------------------


------------------------------------------------------------------------------------------------
-- Funcio que retorna una taula amb un registre del tipus typREGISTREESTGENERALS              --
-- Es pot consultar directament amb la comanda: SELECT * FROM TABLE(fnESTADISTIQUESGENERALS); --
-- Permet obtenir els segents valors estadstics:                                            --
-- Descarregues totals a la plataforma i els euros que han comportat en ingressos             --
------------------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION fnESTADISTIQUESGENERALS RETURN tblTAULAESTGENERALS
IS
  fnESTADISTIQUESGENERALSSRETURN tblTAULAESTGENERALS := tblTAULAESTGENERALS();
  regEstGen typREGISTREESTGENERALS := typREGISTREESTGENERALS(NULL,NULL);
BEGIN

  SELECT DESCARREGUES,EUROSDESCARREGUES INTO regEstGen.descarregues, regEstGen.eurosDescarregues FROM ESTADISTIQUESGENERALS;
  fnESTADISTIQUESGENERALSSRETURN.extend;
  fnESTADISTIQUESGENERALSSRETURN(fnESTADISTIQUESGENERALSSRETURN.count) := regEstGen;
  RETURN fnESTADISTIQUESGENERALSSRETURN;
END;
/

--------------------------------------------------------------------------------------------------------
-- Funcio que retorna una taula amb un registre del tipus typREGISTREESTANUALS                        --
-- T un parametre d'entrada que s l'any que es vol consultar                                        --
-- Es pot consultar directament amb la comanda: SELECT * FROM TABLE(fnESTADISTIQUESANUALS(strAny));   --
-- Permet obtenir els segents valors estadstics per l'any passat com a parametre:                   --
-- Any, mitjana de descarregues per usuari, desenvolupador o desenvolupadors amb ms descarregues,    --
-- el nombre de descarregues d'aquests desenvolupador, l'aplicaci o aplicacions amb ms recaptaci i --
-- els desenvolupadors d'aquestes aplicacions.                                                        --
-- NOTA: els desenvolupadors, si hi ha ms d'un, es retornen separats pel caracter ';'                --
--       les aplicacions, si hi ha ms d'una, es retornen separades pel caracter ';'                  --
--       si hi ha ms d'una aplicaci amb mxima recaptacio, els desenvolupadors d'aquestes es        --
--       retornen en el seguent format des1App1;des2App1;des3App1|des1App2;des2App2                   --
--       s a dir, el separador entre desenvolupadors de la mateixa aplicaci s ';'                  --
--       i el separador entre desenvolupadors de diferents aplicacions s '|'                         --
--------------------------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION fnESTADISTIQUESANUALS (strAny IN CHAR) RETURN tblTAULAESTANUALS
IS
  fnESTADISTIQUESANUALSRETURN tblTAULAESTANUALS := tblTAULAESTANUALS();
  regEstGenAnuals typREGISTREESTANUALS := typREGISTREESTANUALS(SUBSTR(strAny,1,4),NULL,NULL,NULL,NULL,NULL);
BEGIN

  SELECT MITJANADESCARREGUES,DESENVOLUPADORMAXDESC,DESENVOLUPADORMAXDESCNUMERO,APLICACIOMAXRECAPTACIO,APLICACIOMAXRECAPTACIODES 
    INTO regEstGenAnuals.mitjanaDescarregues,regEstGenAnuals.desenvolupadorMaxDesc,regEstGenAnuals.desenvolupadorMaxDescNumero,
    regEstGenAnuals.aplicacioMaxRecapatacio,regEstGenAnuals.aplicacioMaxRecaptacioDes
    FROM ESTADISTIQUESANUALS
    WHERE ANYESTADISTICA=strAny;
  fnESTADISTIQUESANUALSRETURN.extend;
  fnESTADISTIQUESANUALSRETURN(fnESTADISTIQUESANUALSRETURN.count) := regEstGenAnuals;
  RETURN fnESTADISTIQUESANUALSRETURN;
END;
/

--------------------------------------------------------------------------------------------------------------------
-- Funcio que retorna una taula amb un registre del tipus typREGISTREESTANUALSPAIS                                --
-- T dos parametres d'entrada, l'any i el codi ISO 3166-1 alfa-2 del pais                                        --
-- Es pot consultar directament amb la comanda: SELECT * FROM TABLE(fnESTADISTIQUESANUALSPAIS(strAny,strPais));   --
-- Permet obtenir els segents valors estadstics per l'any i el pais passats com a parametres:                   --
-- Any, pais, nombre d'usuaris diferents que han realitzat descarregues, els ingressos que han generat            --
-- els usuaris registrats en aquell pais i les aplicacions diferents que s'han descarregat                        --
--------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION fnESTADISTIQUESANUALSPAIS (strAny IN CHAR, strPais IN CHAR) RETURN tblTAULAESTANUALSPAIS
IS
  fnESTANUALSPAISRETURN tblTAULAESTANUALSPAIS := tblTAULAESTANUALSPAIS();
  regEstGenAnualsPais typREGISTREESTANUALSPAIS := typREGISTREESTANUALSPAIS(SUBSTR(strAny,1,4),SUBSTR(strPais,1,2),NULL,NULL,NULL);
BEGIN

  SELECT USUARISDIF,INGRESSOSUSUARIS,APLICACIONSDIF 
    INTO regEstGenAnualsPais.usuarisDif,regEstGenAnualsPais.ingressosUsuaris,regEstGenAnualsPais.aplicacionsDif
    FROM ESTADISTIQUESANUALSPAIS
    WHERE (ANYESTADISTICA=strAny) AND (PAIS=strPais);
  fnESTANUALSPAISRETURN.extend;
  fnESTANUALSPAISRETURN(fnESTANUALSPAISRETURN.count) := regEstGenAnualsPais;
  RETURN fnESTANUALSPAISRETURN;
END;
/
