-------------------------------------------------------------------------------------------------
-- CREACIO DELS TIPUS NECESSARIS PELS PARAMETRES DE RETORN DELS PROCEDIMENTS DE LA PLATAFORMA  --
-------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat de tots els desenvolupadors d'un pais --
-- Necessaris pel procediment emmagatzemat proLlistatDesPais                       --
-------------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typREGISTRELLISDESPAIS IS OBJECT 
(
   codi NUMBER, 
	 idDesenvolupador VARCHAR2(50 CHAR),
   dataAlta TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   nomEmpresa VARCHAR2 (50 CHAR),
   nomRepLegal VARCHAR2 (50 CHAR),
   adrea VARCHAR2 (200 CHAR),
   telefon VARCHAR2 (20 CHAR),
   appPublicades NUMBER
);
/

CREATE OR REPLACE TYPE tblTAULALLISDESPAIS AS TABLE OF typREGISTRELLISDESPAIS;
/

--------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat de totes les aplicacions actives --
-- Necessaris pel procediment emmagatzemat proLlistatAppActives               --
--------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typREGISTRELLISAPPACTIVES IS OBJECT 
(
   codi NUMBER, 
	 idAplicacio VARCHAR2(50 CHAR),
   dataPujada TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   versio VARCHAR2 (10 CHAR),
   linkVideo VARCHAR2 (50 CHAR),
   resolucioMinima VARCHAR2 (10 CHAR),
   descarregues NUMBER
);
/

CREATE OR REPLACE TYPE tblTAULALLISAPPACTIVES AS TABLE OF typREGISTRELLISAPPACTIVES;
/

-------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat dels paisos on s'ha descarregat --
-- una aplicaci en un any concret                                           --
-- Necessaris pel procediment emmagatzemat proLlistatAppAny                  --
-------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typREGISTRELLISAPPANY IS OBJECT 
(
   pais  VARCHAR2(50 CHAR), 
   descarregues NUMBER
);
/

CREATE OR REPLACE TYPE tblTAULALLISAPPANY AS TABLE OF typREGISTRELLISAPPANY;
/

-------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat de les descarregues d'un usuari --
-- Necessaris pel procediment emmagatzemat proLlistatDescUsuari              --
-------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typREGISTRELLISDESCUSU IS OBJECT 
(
  dispositiuIMEI VARCHAR2(30 CHAR),
  dispositiuModel VARCHAR2(50 CHAR),
  aplicacio VARCHAR2(50 CHAR),
  dataHora TIMESTAMP WITH LOCAL TIME ZONE,
  modePagament VARCHAR2(30 CHAR),
  operador VARCHAR2(50 CHAR),
  preu NUMBER(4,2),
  pais VARCHAR2(50 CHAR)
);
/

CREATE OR REPLACE TYPE tblTAULALLISDESCUSU AS TABLE OF typREGISTRELLISDESCUSU;
/

-------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat de les descarregues d'un usuari --
-- Necessaris pel procediment emmagatzemat proLlistatTop20Usuari             --
-------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typREGISTRELLISTOP20USU IS OBJECT 
(
   codi NUMBER, 
   dataAlta TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   mobil VARCHAR2 (20 CHAR),
   operador VARCHAR2 (50 CHAR),
   email VARCHAR2 (50 CHAR),
   pais VARCHAR2(50 CHAR),
   despesa NUMBER
);
/

CREATE OR REPLACE TYPE tblTAULALLISTOP20USU AS TABLE OF typREGISTRELLISTOP20USU;
/

--------------------------------------------------------------
-- Tipus amb les dades d'un usuari                          --
-- Necessari pel procediment emmagatzemat proConsultaUsuari --
--------------------------------------------------------------
CREATE OR REPLACE TYPE typUSUARI IS OBJECT 
(
   codi NUMBER, 
   dataAlta TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   mobil VARCHAR2 (20 CHAR),
   operador VARCHAR2 (50 CHAR),
   email VARCHAR2 (50 CHAR),
   pais VARCHAR2(50 CHAR)
);
/

-----------------------------------------------------------------
-- Tipus amb les dades d'una aplicaci                         --
-- Necessari pel procediment emmagatzemat proConsultaAplicacio --
-----------------------------------------------------------------
CREATE OR REPLACE TYPE typAPLICACIO IS OBJECT 
(
  codi NUMBER, 
  idAplicacio  VARCHAR2(50 CHAR),
  dataPujada TIMESTAMP WITH LOCAL TIME ZONE,
  dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
  versio VARCHAR2 (10 CHAR),
  linkVideo VARCHAR2 (50 CHAR),   
  resolucioMinima VARCHAR2(10 CHAR),
  activa CHAR(1 CHAR)
);
/

---------------------------------------------------------------------------
-- Tipus amb la llista de sistemes operatius suportats per una aplicaci --
-- Necessari pel procediment emmagatzemat proConsultaAppSO               --
---------------------------------------------------------------------------
CREATE OR REPLACE TYPE tblTAULALLISTSOAPP AS TABLE OF VARCHAR2(50 CHAR);
/

-------------------------------------------------------------------------------------------------------------------
-- Tipus necessaris per a obtenir el llistat dels binaris corresponents a una aplicaci                          --
-- Necessaris pels procediments emmagatzemats proConsultaBinariApp, proLlistatBinarisApp i proConsultaBinarisDes --
-------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typBINARI IS OBJECT 
(
   codi NUMBER, 
   aplicacio VARCHAR2(50 CHAR),
   so VARCHAR2(50 CHAR),
   versio VARCHAR2(10 CHAR),
   dataPujada TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   hipervincle VARCHAR2(50 CHAR),
   mida NUMBER,
   resolucio VARCHAR2(10 CHAR),
   activa CHAR(1 CHAR)
);
/

CREATE OR REPLACE TYPE tblLLISTATBINARISAPP AS TABLE OF typBINARI;
/

------------------------------------------------------------------------------------------------
-- Tipus amb les dades d'un desenvolupador                                                    --
-- Necessari pel procediment emmagatzemat proConsultaDesenvolupador i proConsultaDesBinari    --
------------------------------------------------------------------------------------------------
CREATE OR REPLACE TYPE typDESENVOLUPADOR IS OBJECT 
(
   codi NUMBER, 
	 idDesenvolupador VARCHAR2(50 CHAR),
   dataAlta TIMESTAMP WITH LOCAL TIME ZONE,
   dataBaixa TIMESTAMP WITH LOCAL TIME ZONE,
   nomEmpresa VARCHAR2 (50 CHAR),
   nomRepLegal VARCHAR2 (50 CHAR),
   adrea VARCHAR2 (200 CHAR),
   telefon VARCHAR2 (20 CHAR),
   pais VARCHAR2(50 CHAR)
);
/

CREATE OR REPLACE TYPE tblLLISTATDESENVOLUPADORS AS TABLE OF typDESENVOLUPADOR;
/

------------------------------------------------------------------
-- Tipus amb les dades d'un dispositiu                          --
-- Necessari pel procediment emmagatzemat proConsultaDispUsuari --
------------------------------------------------------------------
CREATE OR REPLACE TYPE typDISPOSITIU IS OBJECT 
(
   codi NUMBER, 
	 IMEI VARCHAR2(30 CHAR),
   modelDisp VARCHAR2(50 CHAR),
   resolucio VARCHAR2(10 CHAR),
   SO VARCHAR2(50 CHAR)
);
/

CREATE OR REPLACE TYPE tblLLISTATDISPOSITIUS AS TABLE OF typDISPOSITIU;
/


-------------------------------------------------
-- CREACIO DELS PROCEDIMENTS DE LA PLATAFORMA  --
-------------------------------------------------


-------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                registreLOG                                                                                                                                     --
-- DESCRIPCIO:         Inserir un registre a la taula de LOG                                                                                                           --
-- PARAMETRES ENTRADA: nomProcediment (nom del procediment que s'ha executat i que s'ha de registrar al LOG)                                                           --
--                     paramsIn (parametres d'entrada que s'han emprat en l'execuci del procediment en el format NomParam1:Valor|NomParam2:Valor|NomParam3:Valor...   --
--                     paramsOut (parametres de sortida que s'han emprat en l'execuci del procediment en el format NomParam1:Valor|NomParam2:Valor|NomParam3:Valor... --
--                     resultat (resultat de l'execuci del procediment nomProcediment: 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                   --
-- PARAMETRES SORTIDA: RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                                                       --
-- EXCEPCIONS:         Cap                                                                                                                                             --
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE registreLOG (nomProcediment IN TLOG.PROCEDIMENT%TYPE,paramsIn IN TLOG.PARINPUT%TYPE,paramsOut IN TLOG.PAROUTPUT%TYPE,resultat IN TLOG.RESULTAT%TYPE,RSP OUT NOCOPY VARCHAR2) IS 
BEGIN

  -- afegim a la taula de log el registre corresponent
  INSERT INTO TLOG (PROCEDIMENT,PARINPUT,PAROUTPUT,RESULTAT) VALUES (nomProcediment,paramsIn,paramsOut,resultat);
  RSP := 'OK';
  COMMIT; -- consolidem les operacions a la base de dades
  
END;
/

------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaAplicacio                                                                           --
-- DESCRIPCIO:             Donar d'alta una aplicacio                                                                 --
-- PARAMETRES ENTRADA:     strIdAplicacio (nom de l'apliacio)                                                         --
--                         strVersio (versio de l'aplicacio)                                                          --
--                         strlinkVideo (link al video de demostracio de l'aplicacio)                                 --
--                         intResolucio (codi de la resolucio minima acceptada per l'aplicacio)                       --
-- PARAMETRES SORTIDA:     intCodiAplicacio (codi de l'aplicacio acabada de donar d'alta)                             --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)  --
-- EXCEPCIONS CONTROLADES: ID_NULL (el parametre strIdAplicacio es NULL)                                              --
--                         ID_EXISTEIX (el parametre strIdAplicacio ja existeix)                                      --
--                         RESOLUCIO_NO_EXISTEIX (el parametre intResolucio no existeix)                              --
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaAplicacio (strIdAplicacio IN APLICACIO.IDAPLICACIO%TYPE,strVersio IN APLICACIO.VERSIO%TYPE, strlinkVideo IN APLICACIO.LINKVIDEO%TYPE,intResolucio IN APLICACIO.RESOLUCIOMINIMA%TYPE,intCodiAplicacio OUT APLICACIO.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
ID_NULL EXCEPTION; -- excepcio corresponent al nom de l'aplicacio amb valor NULL
ID_EXISTEIX EXCEPTION; -- excepcio corresponent al nom de l'aplicacio amb un valor que ja existeix
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de resolucio amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strIdAplicacio IS NULL THEN
    RAISE ID_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE IDAPLICACIO = strIdAplicacio;
  IF intComptador > 0 THEN
    RAISE ID_EXISTEIX;
  END IF;  

  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;  
  END IF;

  -- donem d'alta l'aplicaci
  INSERT INTO APLICACIO (IDAPLICACIO,VERSIO,LINKVIDEO,RESOLUCIOMINIMA) VALUES (strIdAplicacio,strVersio,strlinkVideo,intResolucio);
  SELECT CODI INTO intCodiAplicacio FROM APLICACIO WHERE IDAPLICACIO = strIdAplicacio; -- recuperem el codi de l'aplicacio que s'ha acabat de donar d'alta
  strPARAMOUT := 'intCodiAplicacio:' || intCodiAplicacio; 
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaAplicacio','strIdAplicacio:' || strIdAplicacio || ', strVersio:' || strVersio || ', strlinkVideo:' || strlinkVideo || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
  
  --gesti de les excepcions
  EXCEPTION
    WHEN ID_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strIdAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAplicacio','strIdAplicacio:' || strIdAplicacio || ', strVersio:' || strVersio || ', strlinkVideo:' || strlinkVideo || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN ID_EXISTEIX THEN
      RSP := 'ERROR: l''aplicaci amb nom ' || strIdAplicacio || ' ja existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAplicacio','strIdAplicacio:' || strIdAplicacio || ', strVersio:' || strVersio || ', strlinkVideo:' || strlinkVideo || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de resoluci ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAplicacio','strIdAplicacio:' || strIdAplicacio || ', strVersio:' || strVersio || ', strlinkVideo:' || strlinkVideo || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proAltaAplicacio','strIdAplicacio:' || strIdAplicacio || ', strVersio:' || strVersio || ', strlinkVideo:' || strlinkVideo || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proBaixaAplicacio                                                                                                 --
-- DESCRIPCIO:             Donar de baixa una aplicacio pel seu codi o b pel seu nom (preval el codi si esta informat)                      --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicaci que es vol donar de baixa, preval sobre l'altre parametre d'entrada strNom) --
--                         strNom (nom de l'aplicaci, si intCodiAplicacio no es NULL aquest parametre es ignorat)                           --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                         --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio i strNom son NULL)                                                  --
--                         CODI_NO_EXISTEIX (no existeix cap aplicacio amb el codi igual a intCodiAplicacio)                                 --
--                         NOM_NO_EXISTEIX (no existeix cap aplicacio amb el nom igual a strNom)                                             --
--                         APLICACIO_DE_BAIXA (l'aplicaci ja estava donada de baixa)                                                        --
------------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proBaixaAplicacio (intCodiAplicacio IN APLICACIO.CODI%TYPE,strNom IN APLICACIO.IDAPLICACIO%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intCodiApliacioTemp NUMBER; -- variable per a recollir el codi de l'aplicacio quan s'ha informat el nom
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio d'entrada i nom amb valor NULL
CODI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicaci d'entrada amb un valor que no existeix
NOM_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al nom de l'aplicacio d'entrada amb un valor que no existeix
APLICACIO_DE_BAIXA EXCEPTION; -- excepcio corresponent a que l'aplicaci que es vol donar de baixa ja ho esta

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF (intCodiAplicacio IS NULL) AND (strNom IS NULL) THEN
    RAISE APLICACIO_NULL;
  END IF;

  IF NOT (intCodiAplicacio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
    IF intComptador = 0 THEN
      RAISE CODI_NO_EXISTEIX;
    ELSE
      intCodiApliacioTemp := intCodiAplicacio;
    END IF;    
  ELSE 
    SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE IDAPLICACIO = strNom;
    IF intComptador = 0 THEN
      RAISE NOM_NO_EXISTEIX;
    ELSE
      SELECT CODI INTO intCodiApliacioTemp FROM APLICACIO WHERE IDAPLICACIO = strNom;
    END IF;      
  END IF;

  -- comprovem que l'aplicacio no estigui donada de baixa
  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE (CODI=intCodiApliacioTemp) AND (NOT (DATABAIXA IS NULL));
  IF intComptador > 0 THEN
    RAISE APLICACIO_DE_BAIXA;
  END IF;  

  -- actualitzem la data de baixa de l'aplicaci i la posem com a no activa
  UPDATE APLICACIO SET DATABAIXA=SYSTIMESTAMP,ACTIVA='N' WHERE CODI=intCodiApliacioTemp;
  -- registrem l'execuci del procediment al log
  registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);
  
  --gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: els parmetres d''entrada intCodiAplicacio i strNom sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);

    WHEN CODI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap aplicaci amb el codi ' || intCodiAplicacio;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);

    WHEN NOM_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap aplicaci amb el nom ' || strNom;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_DE_BAIXA THEN
      RSP := 'ERROR: l''aplicaci ja estava donada de baixa';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);
      
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proBaixaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strNom:' || strNom,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proModificaAplicacio                                                                                           --
-- DESCRIPCIO:             Modificar les dades d'una aplicaci. Si un parametre de modificaci s NULL s'ignora,                          --
--                         no es modifiquen camps amb valors NULL                                                                         --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicaci del qual es volen modificar les dades)                                   --
--                         strVersio (nou codi de la versio de l'aplicacio)                                                               --
--                         strLinkVideo (nou link al video de demostracio de l'aplicacio)                                                 --
--                         intResolucio (nou codi de la resolucio minima acceptada per l'aplicacio)                                       --
--                         strActiva (nou valor que indica si l'aplicacio esta activa o no per a descarregar, valors acceptats 'S' i 'N') --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                      --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                                         --
--                         APLICACIO_NO_EXISTEIX (no existeix cap aplicacio amb codi igual a intCodiAplicacio)                            --
--                         RESOLUCIO_NO_EXISTEIX (el parametre intResolucio no existeix)                                                  --
--                         ACTIVA_INCORRECTE (el parametre strActiva cont un valor incorrecte, diferent de 'S' i 'N')                    --
--                         VALORS_NULL (tots els camps a modificar sn NULL)                                                              --
--------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proModificaAplicacio (intCodiAplicacio IN APLICACIO.CODI%TYPE,strVersio IN APLICACIO.VERSIO%TYPE, strLinkVideo IN APLICACIO.LINKVIDEO%TYPE,
                                                  intResolucio IN APLICACIO.RESOLUCIOMINIMA%TYPE,strActiva IN APLICACIO.ACTIVA%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strSeparador VARCHAR2 (1 CHAR); -- per afegir el separador dels camps a modificar en la construccio de la sentencia SQL que s'ha d'executar
strSQL VARCHAR2(500 CHAR); -- per construir la sentencia SQL que s'ha d'executar
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'apllicacio d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio  d'entrada amb un valor que no existeix
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de resolucio d'entrada amb un valor que no existeix
ACTIVA_INCORRECTE EXCEPTION; -- excepcio corresponent al codi d'aplicacio activa o no amb un valor incorrecte, diferent de 'S' i 'N'
VALORS_NULL EXCEPTION; -- excepcio corresponent a que tots els valors a modificar sn NULL, s a dir, no hi ha res a modificar

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  
  
  IF (strVersio IS NULL) AND (strLinkVideo IS NULL) AND (intResolucio IS NULL) AND (strActiva IS NULL) THEN
    RAISE VALORS_NULL;
  END IF;
  
  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;
  END IF;

  IF NOT (strActiva IS NULL) THEN
    IF (strActiva <> 'S') AND (strActiva <> 'N') THEN
      RAISE ACTIVA_INCORRECTE;
    END IF;
  END IF;
  
  -- Construim la sentencia SQL d'actualitzaci que s'ha d'executar
  strSeparador := '';
  strSQL := 'UPDATE APLICACIO SET ';
  
  IF NOT (strVersio IS NULL) THEN
    strSQL := strSQL || ' VERSIO = ''' || strVersio || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strLinkVideo IS NULL) THEN
    strSQL := strSQL || strSeparador || ' LINKVIDEO = ''' || strLinkVideo || '''';
    strSeparador := ',';
  END IF;

  IF NOT (intResolucio IS NULL) THEN
    strSQL := strSQL || strSeparador || ' RESOLUCIOMINIMA = ' || intResolucio ;
    strSeparador := ',';
  END IF;
  
  IF NOT (strActiva IS NULL) THEN
    strSQL := strSQL || strSeparador || ' ACTIVA = ''' || strActiva || '''';
  END IF;
  
  strSQL := strSQL || ' WHERE CODI = ' || intCodiAplicacio;
  -- executem la sentncia SQL
  EXECUTE IMMEDIATE strSQL;
  -- registrem l'execuci del procediment al log
  registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
                
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap aplicaci amb el codi ' || intCodiAplicacio;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
  
    WHEN VALORS_NULL THEN
      RSP := 'ERROR: tots els camps sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de la resoluci mnima ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN ACTIVA_INCORRECTE THEN
      RSP := 'ERROR: el codi que indica si l''aplicaci esta activa o no s incorrecte, diferent de ''S'' i ''N''';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaAplicacio','intCodiAplicacio:' || intCodiAplicacio || ', strVersio:' || strVersio || ', strLinkVideo:' || strLinkVideo 
                || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaAplicacio                                                                         --
-- DESCRIPCIO:             Obtenir totes les dades d'una aplicaci mitjanant un objecte de tipus typAPLICACIO          --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicaci de la qual es volen obtenir les dades)                 --
-- PARAMETRES SORTIDA:     typDadesAplicacio (objecte de tipus typAPLICACIO amb la informaci de l'aplicaci)          --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)    --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el codi de l'aplicaci passada com a parametre d'entrada es NULL)            --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicaci passada com a parametre d'entrada no existeix) --
--------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaAplicacio (intCodiAplicacio IN APLICACIO.CODI%TYPE,typDadesAplicacio OUT typAPLICACIO,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicaci d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicaci d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;
  
  strPARAMOUT := '';
  typDadesAplicacio := typAPLICACIO(intCodiAplicacio,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a inicialitzar l'objecte amb les dades de l'aplicaci
  
  -- recuperem les dades de l'aplicaci i les guardem a l'objecte typDadesAplicacio
  SELECT APLICACIO.CODI,APLICACIO.IDAPLICACIO,APLICACIO.DATAPUJADA,APLICACIO.DATABAIXA,APLICACIO.VERSIO,APLICACIO.LINKVIDEO,RESOLUCIO.NOM,APLICACIO.ACTIVA 
  INTO typDadesAplicacio.codi,typDadesAplicacio.idAplicacio,typDadesAplicacio.dataPujada,typDadesAplicacio.dataBaixa,typDadesAplicacio.versio,typDadesAplicacio.linkVideo,typDadesAplicacio.resolucioMinima,typDadesAplicacio.activa 
  FROM APLICACIO LEFT JOIN RESOLUCIO ON APLICACIO.RESOLUCIOMINIMA=RESOLUCIO.CODI 
  WHERE APLICACIO.CODI=intCodiAplicacio;

  strPARAMOUT := 'codi:' || typDadesAplicacio.codi || ',' 
                  || 'idAplicacio:' || typDadesAplicacio.idAplicacio || ',' 
                  || 'dataPujada:' || typDadesAplicacio.dataPujada || ',' 
                  || 'dataBaixa:' || typDadesAplicacio.dataBaixa || ',' 
                  || 'versio:' || typDadesAplicacio.versio || ',' 
                  || 'linkVideo:' || typDadesAplicacio.linkVideo || ',' 
                  || 'resolucioMinima:' || typDadesAplicacio.resolucioMinima || ',' 
                  || 'activa:' || typDadesAplicacio.activa;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      registreLOG('proConsultaAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: l''aplicaci amb codi ' || intCodiAplicacio || ' no existeix';
      registreLOG('proConsultaAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
END;
/

----------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaAppPreuPais                                                                                   --
-- DESCRIPCIO:             Donar d'alta el preu d'una aplicacio per a un pais determinat, si per aquest pais ja existia un preu --
--                         aquest es modificar pel nou                                                                          --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                                 --
--                         intPreu (import de l'aplicacio per al pais determinat en el parametre strPais)                       --
--                         strPais (codi ISO 3166-1 alfa-2 del pais)                                                            --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)            --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                               --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                           --
--                         PREU_NULL (el parametre intPreu es NULL)                                                             --
--                         PAIS_NULL (el parametre strPais es NULL)                                                             --
--                         PAIS_NO_EXISTEIX (el codi del pais no existeix)                                                      --
----------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaAppPreuPais (intCodiAplicacio IN APLICACIO.CODI%TYPE,intPreu IN APLICACIOPREU.PREU%TYPE, strPais IN PAIS.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
PREU_NULL EXCEPTION; -- excepcio corresponent al preu amb valor NULL
PAIS_NULL EXCEPTION; -- excepcio corresponent al codi del pais amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del pais amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF intPreu IS NULL THEN
    RAISE PREU_NULL;
  END IF;
  
  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;  
  
  -- fem la distinci de dos casos, si l'aplicacio ja tenia preu o no
  SELECT COUNT(*) INTO intComptador FROM APLICACIOPREU WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  IF intComptador = 0 THEN
    INSERT INTO APLICACIOPREU (CODIAPLICACIO,CODIPAIS,PREU) VALUES (intCodiAplicacio,strPais,intPreu);
  ELSE
    UPDATE APLICACIOPREU SET PREU=intPreu WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  END IF;
  
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PREU_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intPreu s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
      
    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strPais s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
      
    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proAltaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', intPreu:' || intPreu || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proEliminaAppPreuPais                                                                     --
-- DESCRIPCIO:             Elimina el preu d'una aplicacio per a un pais determinat                                  --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                      --
--                         strPais (codi ISO 3166-1 alfa-2 del pais)                                                 --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                    --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                --
--                         PAIS_NULL (el parametre strPais es NULL)                                                  --
--                         PAIS_NO_EXISTEIX (el codi del pais no existeix)                                           --
--                         PREU_NO_EXISTEIX (no existeix preu per a l'aplicacio i el pais)                           --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proEliminaAppPreuPais (intCodiAplicacio IN APLICACIO.CODI%TYPE,strPais IN PAIS.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
PAIS_NULL EXCEPTION; -- excepcio corresponent al codi del pais amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del pais amb un valor que no existeix
PREU_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que no existeix preu per a l'aplicacio i el pais

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM APLICACIOPREU WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  IF intComptador = 0 THEN
    RAISE PREU_NO_EXISTEIX;
  END IF;  
  
  -- esborrem el preu de l'aplicaci pel pas donat
  DELETE FROM APLICACIOPREU WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  -- registrem l'execuci del procediment al log
  registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strPais s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
      
    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PREU_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix preu per a l''aplicaci amb codi ' || intCodiAplicacio || ' i pais amb codi ' || strPais;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proEliminaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaAppPreuPais                                                                    --
-- DESCRIPCIO:             Retorna el preu d'una aplicaci per a un pais determinat                                  --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                      --
--                         strPais (codi ISO 3166-1 alfa-2 del pais)                                                 --
-- PARAMETRES SORTIDA:     intPreu (preu de l'aplicacio pel pais determinat a l'entrada)                             --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                    --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                --
--                         PAIS_NULL (el parametre strPais es NULL)                                                  --
--                         PAIS_NO_EXISTEIX (el codi del pais no existeix)                                           --
--                         PREU_NO_EXISTEIX (no existeix preu per a l'aplicacio i el pais)                           --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaAppPreuPais (intCodiAplicacio IN APLICACIO.CODI%TYPE,strPais IN PAIS.CODI%TYPE,intPreu OUT APLICACIOPREU.PREU%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
PAIS_NULL EXCEPTION; -- excepcio corresponent al codi del pais amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del pais amb un valor que no existeix
PREU_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que no existeix preu per a l'aplicacio i el pais

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM APLICACIOPREU WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  IF intComptador = 0 THEN
    RAISE PREU_NO_EXISTEIX;
  END IF;  
  
  -- recuperem el preu de l'aplicaci pel pais donat
  SELECT PREU INTO intPreu FROM APLICACIOPREU WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIPAIS = strPais);
  strPARAMOUT := 'intPreu:' || intPreu; 
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strPais s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
      
    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PREU_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix preu per a l''aplicaci amb codi ' || intCodiAplicacio || ' i pais amb codi ' || strPais;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppPreuPais','intCodiAplicacio:' || intCodiAplicacio || ', strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

-------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaAppDescIdioma                                                                                    --
-- DESCRIPCIO:             Donar d'alta la descripcio d'una aplicacio per a un idioma determinat, si aquesta descripcio ja existia --
--                         no dona cap error i es modificada per la nova                                                           --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                                    --
--                         strDescripcio (descripcio de l'aplicacio per a l'idioma determinat en el parametre strIdioma)           --
--                         intIdioma (codi de l'idioma)                                                                            --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)               --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                                  --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                              --
--                         DESCRIPCIO_NULL (el parametre intDescripcio es NULL)                                                    --
--                         IDIOMA_NULL (el parametre intIdioma es NULL)                                                            --
--                         IDIOMA_NO_EXISTEIX (el codi de l'idioma no existeix)                                                    --
-------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaAppDescIdioma (intCodiAplicacio IN APLICACIO.CODI%TYPE,strDescripcio IN APLICACIODESCRIPCIO.DESCRIPCIO%TYPE, intIdioma IN IDIOMA.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
DESCRIPCIO_NULL EXCEPTION; -- excepcio corresponent a la descripcio amb valor NULL
IDIOMA_NULL EXCEPTION; -- excepcio corresponent al codi de l'idioma amb valor NULL
IDIOMA_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'idioma amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF strDescripcio IS NULL THEN
    RAISE DESCRIPCIO_NULL;
  END IF;
  
  IF intIdioma IS NULL THEN
    RAISE IDIOMA_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM IDIOMA WHERE CODI = intIdioma;
  IF intComptador = 0 THEN
    RAISE IDIOMA_NO_EXISTEIX;
  END IF;  
  
  -- fem la distinci de dos casos, si l'aplicacio ja tenia descripcio o no
  SELECT COUNT(*) INTO intComptador FROM APLICACIODESCRIPCIO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  IF intComptador = 0 THEN
    INSERT INTO APLICACIODESCRIPCIO (CODIAPLICACIO,CODIIDIOMA,DESCRIPCIO) VALUES (intCodiAplicacio,intIdioma,strDescripcio);
  ELSE
    UPDATE APLICACIODESCRIPCIO SET DESCRIPCIO=strDescripcio WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  END IF;
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN DESCRIPCIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strDescripcio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
      
    WHEN IDIOMA_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intIdioma s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
      
    WHEN IDIOMA_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de l''idioma ' || intIdioma || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proAltaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', strDescripcio:' || strDescripcio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
END;
/

------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proEliminaAppDescIdioma                                                                    --
-- DESCRIPCIO:             Elimina la descripcio d'una aplicacio per a un idioma determinat                           --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                       --
--                         intIdioma (codi de l'idioma)                                                               --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)  --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                     --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                 --
--                         IDIOMA_NULL (el parametre intIdioma es NULL)                                               --
--                         IDIOMA_NO_EXISTEIX (el codi de l'idioma no existeix)                                       --
--                         DESCRIPCIO_NO_EXISTEIX (no existeix la descripcio per a l'aplicacio i l'idioma determinat) --
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proEliminaAppDescIdioma (intCodiAplicacio IN APLICACIO.CODI%TYPE,intIdioma IN IDIOMA.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
IDIOMA_NULL EXCEPTION; -- excepcio corresponent al codi de l'idioma amb valor NULL
IDIOMA_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'idioma amb un valor que no existeix
DESCRIPCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que no existeix cap descripcio per a l'aplicacio i l'idioma d'entrada

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF intIdioma IS NULL THEN
    RAISE IDIOMA_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM IDIOMA WHERE CODI = intIdioma;
  IF intComptador = 0 THEN
    RAISE IDIOMA_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM APLICACIODESCRIPCIO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  IF intComptador = 0 THEN
    RAISE DESCRIPCIO_NO_EXISTEIX;
  END IF;  
  
  -- esborrem la descripci de l'aplicaci per l'idioma donat
  DELETE FROM APLICACIODESCRIPCIO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  -- registrem l'execuci del procediment al log
  registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN IDIOMA_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intIdioma s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
      
    WHEN IDIOMA_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de l''idioma ' || intIdioma || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN DESCRIPCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix descripci per a l''aplicaci amb codi ' || intCodiAplicacio || ' i idioma amb codi ' || intIdioma;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proEliminaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaAppDescIdioma                                                                  --
-- DESCRIPCIO:             Retorna la descripci d'una aplicaci per a un idioma determinat                          --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                      --
--                         intIdioma (codi de l'idioma del qual es vol obtenir la descripci de l'aplicacio)         --
-- PARAMETRES SORTIDA:     strDescripcio (descripci de l'aplicacio per l'idioma determinat a l'entrada)             --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                    --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                --
--                         IDIOMA_NULL (el parametre intIdioma es NULL)                                              --
--                         IDIOMA_NO_EXISTEIX (el codi de l'idioma no existeix)                                      --
--                         DESCRIPCIO_NO_EXISTEIX (no existeix descripci de l'aplicaci per l'idioma sollicitat)   --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaAppDescIdioma (intCodiAplicacio IN APLICACIO.CODI%TYPE,intIdioma IN IDIOMA.CODI%TYPE,strDescripcio OUT APLICACIODESCRIPCIO.DESCRIPCIO%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
IDIOMA_NULL EXCEPTION; -- excepcio corresponent al codi de l'idioma amb valor NULL
IDIOMA_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'idioma amb un valor que no existeix
DESCRIPCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que no existeix descripcio de l'aplicacio per l'idioma sollicitat

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF intIdioma IS NULL THEN
    RAISE IDIOMA_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM IDIOMA WHERE CODI = intIdioma;
  IF intComptador = 0 THEN
    RAISE IDIOMA_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM APLICACIODESCRIPCIO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  IF intComptador = 0 THEN
    RAISE DESCRIPCIO_NO_EXISTEIX;
  END IF;  
  
  -- obtenim la descripci de l'aplicaci per l'idioma donat
  SELECT DESCRIPCIO INTO strDescripcio FROM APLICACIODESCRIPCIO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODIIDIOMA = intIdioma);
  strPARAMOUT := 'strDescripcio:' || strDescripcio; 
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN IDIOMA_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intIdioma s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
      
    WHEN IDIOMA_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de l''idioma ' || intIdioma || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN DESCRIPCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix descripci per a l''aplicaci amb codi ' || intCodiAplicacio || ' i l''idioma amb codi ' || intIdioma;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proConsultaAppDescIdioma','intCodiAplicacio:' || intCodiAplicacio || ', intIdioma:' || intIdioma,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaAppSO                                                                                 --
-- DESCRIPCIO:             Donar d'alta la vinculaci entre una aplicacio i un sistema operatiu que suporta, si aquesta --
--                         vinculaci ja existeix, no fa res ni retorna cap codi d'error                                --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                         --
--                         strSO (codi del sistema operatiu que es vol definir com a suportat per l'aplicacio)          --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)    --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                       --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                   --
--                         SO_NULL (el parametre strSO es NULL)                                                         --
--                         SO_NO_EXISTEIX (el codi del sistema operatiu no existeix)                                    --
--------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaAppSO (intCodiAplicacio IN APLICACIO.CODI%TYPE,strSO IN SO.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
SO_NULL EXCEPTION; -- excepcio corresponent al codi del sistema operatiu amb valor NULL
SO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del sistema operatiu amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF strSO IS NULL THEN
    RAISE SO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM SO WHERE CODI = strSO;
  IF intComptador = 0 THEN
    RAISE SO_NO_EXISTEIX;
  END IF;  
  
  -- fem la distinci de dos casos, si l'aplicacio ja tenia la vinculacio amb el sistema operatiu (no fer res) o no
  SELECT COUNT(*) INTO intComptador FROM APLICACIOSO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODISO = strSO);
  IF intComptador = 0 THEN
    INSERT INTO APLICACIOSO (CODIAPLICACIO,CODISO) VALUES (intCodiAplicacio,strSO);
  END IF;
  
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN SO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strSO s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
      
    WHEN SO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del sistema operatiu ' || strSO || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proAltaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
END;
/

-------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proEliminaAppSO                                                                             --
-- DESCRIPCIO:             Elimina la vinculaci entre una aplicaci i un sistema operatiu que soporta                 --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi identificador de l'aplicacio)                                        --
--                         strSO (codi del sistema operatiu suportat)                                                  --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)   --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                      --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                  --
--                         SO_NULL (el parametre strSO es NULL)                                                        --
--                         SO_NO_EXISTEIX (el codi del sistema operatiu no existeix)                                   --
--                         VINCULACIO_NO_EXISTEIX (no existeix la vinculacio entre l'aplicacio i el sistema operatiu ) --
-------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proEliminaAppSO (intCodiAplicacio IN APLICACIO.CODI%TYPE,strSO IN SO.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
SO_NULL EXCEPTION; -- excepcio corresponent al codi del sistema operatiu amb valor NULL
SO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del sistema operatiu amb un valor que no existeix
VINCULACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que no existeix cap vinculacio entre el sistema operatiu i l'aplicaci

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  

  IF strSO IS NULL THEN
    RAISE SO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM SO WHERE CODI = strSO;
  IF intComptador = 0 THEN
    RAISE SO_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM APLICACIOSO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODISO = strSO);
  IF intComptador = 0 THEN
    RAISE VINCULACIO_NO_EXISTEIX;
  END IF;  
  
  -- esborrem la vinculaci de l'aplicaci amb el sistema operatiu corresponent
  DELETE FROM APLICACIOSO WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODISO = strSO);
  -- registrem l'execuci del procediment al log
  registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN SO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strSO s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
      
    WHEN SO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del sistema operatiu ' || strSO || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN VINCULACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix la vinculaci entre l''aplicaci amb codi ' || intCodiAplicacio || ' i el sistema operatiu amb codi ' || strSO;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proEliminaAppSO','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO,strPARAMOUT,RSP,strRTN);
END;
/

----------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaAppSO                                                                                                 --
-- DESCRIPCIO:             Obtenir un llistat de tots els sistemes operatius suportats per una aplicaci                                    --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicacio de la qual es vol recuperar els sistemes operatius que suporta)            --
-- PARAMETRES SORTIDA:     tblLlistatSO (taula de tipus tblTAULALLISTSOAPP amb el resultat de la consulta)                                  --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                        --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el codi de l'aplicaci passat com a parametre d'entrada es NULL)                                 --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicaci passat com a parametre d'entrada no existeix a la taula de paisos) --
----------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaAppSO (intCodiAplicacio IN APLICACIO.CODI%TYPE,tblLlistatSO OUT tblTAULALLISTSOAPP,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strDescripcioSO VARCHAR2(50 CHAR);
CURSOR cursorSO IS -- cursor amb els desenvolupadors del pais passat com a entrada
  SELECT NOM FROM APLICACIOSO LEFT JOIN SO ON (APLICACIOSO.CODISO = SO.CODI) WHERE CODIAPLICACIO = intCodiAplicacio ORDER BY NOM; 
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicaci d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicaci d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;
  
  tblLlistatSO := tblTAULALLISTSOAPP();
  strPARAMOUT := '';

  -- es recorren tots els desenvolupadors del cursor buscant per cadascun d'ells les aplicacions desenvolupades formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorSO;
    LOOP 
      FETCH cursorSO
        INTO strDescripcioSO;
        EXIT WHEN cursorSO%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT ||  strDescripcioSO;
        tblLlistatSO.extend; -- afegim un nou registre a la taula
        tblLlistatSO(tblLlistatSO.count) := strDescripcioSO; -- al registre acabat de crear li donem valor
    END LOOP;
	CLOSE cursorSO;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaAppSO','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions  
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      registreLOG('proConsultaAppSO','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''apliaci ' || intCodiAplicacio || ' no existeix';
      registreLOG('proConsultaAppSO','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaAppSO','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaBinariApp                                                                             --
-- DESCRIPCIO:             Donar d'alta un binari associat a una aplicacio                                              --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicacio a la qual es vol associar un nou binari)               --
--                         strSO (codi del sistema operatiu suportat pel binari)                                        --
--                         strVersio (versio del binari)                                                                --
--                         strHipervincle (ubicaci fsica del fitxer binari)                                           --
--                         intMida (tamany en bytes del fitxer binari)                                                  --
--                         intResolucio (codi de la resoluci mnima que admet el binari)                               --
-- PARAMETRES SORTIDA:     intCodiBinari (codi del binari acabat de donar d'alta)                                       --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)    --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                       --
--                         APLICACIO_NO_EXISTEIX (el codi de l'aplicacio no existeix)                                   --
--                         SO_NULL (el parametre strSO es NULL)                                                         --
--                         SO_NO_EXISTEIX (el codi del sistema operatiu no existeix)                                    --
--                         VERSIO_NULL (el parametre strVersio es NULL)                                                 --
--                         HIPERVINCLE_NULL (el parametre strHipervincle es NULL)                                       --
--                         MIDA_NULL (el parametre intMida es NULL)                                                     --
--                         RESOLUCIO_NO_EXISTEIX (el codi de la resolucio no existeix)                                  --
--                         BINARI_EXISTEIX (el fitxer binari ja existeix, mateixa aplicacio, sistema operatiu i versio) --
--------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaBinariApp (intCodiAplicacio IN APLICACIO.CODI%TYPE,strSO IN SO.CODI%TYPE, strVersio IN BINARI.VERSIO%TYPE,strHipervincle IN BINARI.HIPERVINCLE%TYPE,
                                              intMida IN BINARI.MIDA%TYPE,intResolucio IN RESOLUCIO.CODI%TYPE,intCodiBinari OUT BINARI.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio amb un valor que no existeix
SO_NULL EXCEPTION; -- excepcio corresponent al codi del sistema operatiu d'entrada amb valor NULL
SO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del sistema operatiu amb un valor que no existeix
VERSIO_NULL EXCEPTION; -- excepcio corresponent a la versio d'entrada amb valor NULL
HIPERVINCLE_NULL EXCEPTION; -- excepcio corresponent al hipervincle d'entrada amb valor NULL
MIDA_NULL EXCEPTION; -- excepcio corresponent a la mida d'entrada amb valor NULL
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de la resolucio amb un valor que no existeix
BINARI_EXISTEIX EXCEPTION; -- excepcio corresponent a l'existencia d'un binari pel mateix codi d'aplicacio, versio i sistema operatiu

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;  
  
  IF strSO IS NULL THEN
    RAISE SO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM SO WHERE CODI = strSO;
  IF intComptador = 0 THEN
    RAISE SO_NO_EXISTEIX;
  END IF;  

  IF strVersio IS NULL THEN
    RAISE VERSIO_NULL;
  END IF;

  IF strHipervincle IS NULL THEN
    RAISE HIPERVINCLE_NULL;
  END IF;

  IF intMida IS NULL THEN
    RAISE MIDA_NULL;
  END IF;
  
  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;  
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODISO = strSO) AND (VERSIO = strVersio);
  IF intComptador > 0 THEN
    RAISE BINARI_EXISTEIX;
  END IF;  

  -- afegim el nou fitxer binari a la plataforma
  INSERT INTO BINARI (CODIAPLICACIO,CODISO,VERSIO,HIPERVINCLE,MIDA,RESOLUCIO) VALUES (intCodiAplicacio,strSO,strVersio,strHipervincle,intMida,intResolucio);
  SELECT CODI INTO intCodiBinari FROM BINARI WHERE (CODIAPLICACIO = intCodiAplicacio) AND (CODISO = strSO) AND (VERSIO = strVersio); -- recuperem el codi del binari acabat de donar d'alta
  strPARAMOUT := 'intCodiBinari:' || intCodiBinari; 
  -- registrem l'execici del procediment al log
  registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap aplicaci amb el codi ' || intCodiAplicacio;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
                
    WHEN SO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strSO s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN SO_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap sistema operatiu amb el codi ' || strSO;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN VERSIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strVersio s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN HIPERVINCLE_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strHipervincle s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN MIDA_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intMida s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de resoluci ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_EXISTEIX THEN
      RSP := 'ERROR: ja existeix un binari associat a l''aplicaci ' || intCodiAplicacio || ', pel sistema operatiu ' || strSO || ' i amb la versi ' || strVersio;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
                
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaBinariApp','intCodiAplicacio:' || intCodiAplicacio || ', strSO:' || strSO || ', strVersio:' || strVersio || ',strHipervincle:' || strHipervincle
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proBaixaBinariApp                                                                         --
-- DESCRIPCIO:             Donar de baixa un binari d'una aplicaci                                                  --
-- PARAMETRES ENTRADA:     intCodiBinari (codi del fitxer binari que es vol donar de baixa)                          --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: BINARI_NULL (el parametre intCodiBinari s NULL)                                          --
--                         BINARI_NO_EXISTEIX (no existeix cap binari amb el codi igual a intCodiBinari)             --
--                         BINARI_DE_BAIXA (el binari ja estava donat de baixa)                                      --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proBaixaBinariApp (intCodiBinari IN BINARI.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi del fitxer binari amb valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del fitxer binari d'entrada amb un valor que no existeix
BINARI_DE_BAIXA EXCEPTION; -- excepcio corresponent a que el fitxer binari ja estava donat de baixa

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intCodiBinari;
  IF intComptador = 0 THEN
      RAISE BINARI_NO_EXISTEIX;
  END IF;    

  -- comprovem que el binari no estigui donat de baixa
  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE (CODI=intCodiBinari) AND (NOT (DATABAIXA IS NULL));
  IF intComptador > 0 THEN
    RAISE BINARI_DE_BAIXA;
  END IF;  

  -- per donar de baixa el binari actualitzem la data de baixa amb l'actual i el podem com a no actiu
  UPDATE BINARI SET DATABAIXA=SYSTIMESTAMP,ACTIVA='N' WHERE CODI=intCodiBinari;
  -- registrem l'execuci del procediment al log
  registreLOG('proBaixaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excecpions
  EXCEPTION
    WHEN BINARI_NULL THEN
      RSP := 'ERROR: els parmetres d''entrada intCodiBinari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap fitxer binari amb el codi ' || intCodiBinari;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_DE_BAIXA THEN
      RSP := 'ERROR: el binari ja estava donat de baixa';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proBaixaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proModificaBinariApp                                                                                        --
-- DESCRIPCIO:             Modificar les dades d'un fitxer binari associat a una aplicaci. Si un parametre de modificaci s NULL     --
--                         s'ignora, no es modifiquen camps amb valors NULL. No es pot canviar el sistema operatiu ni la versi        --
--                         perque juntament amb el codi de l'aplicaci son camps identificadors del binari                             --
-- PARAMETRES ENTRADA:     intCodiBinari (codi del fitxer binari del qual es volen modificar les dades)                                --
--                         strHipervincle (nou hipervincle al fitxer fsic del binari)                                                 --
--                         intMida (nova mida del fitxer binari en bytes)                                                              --
--                         intResolucio (nou codi de la resolucio admesa pel binari)                                                   --
--                         strActiva (nou valor que indica si el binari esta actiu o no per a descarregar, valors acceptats 'S' i 'N') --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                   --
-- EXCEPCIONS CONTROLADES: BINARI_NULL (el parametre intCodiBinari es NULL)                                                            --
--                         BINARI_NO_EXISTEIX (no existeix cap binari amb codi igual a intCodiBinari)                                  --
--                         RESOLUCIO_NO_EXISTEIX (el parametre intResolucio no correspon a cap codi de resolucio)                      --
--                         ACTIVA_INCORRECTE (el parametre strActiva cont un valor incorrecte, diferent de 'S' i 'N')                 --
--                         VALORS_NULL (tots els camps a modificar sn NULL)                                                           --
-----------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proModificaBinariApp (intCodiBinari IN BINARI.CODI%TYPE,strHipervincle IN BINARI.HIPERVINCLE%TYPE,
                                                  intMida IN BINARI.MIDA%TYPE, intResolucio IN RESOLUCIO.CODI%TYPE,strActiva IN APLICACIO.ACTIVA%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strSeparador VARCHAR2 (1 CHAR); -- per afegir el separador dels camps a modificar en la construccio de la sentencia SQL que s'ha d'executar
strSQL VARCHAR2(500 CHAR); -- per construir la sentencia SQL que s'ha d'executar
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi del binari d'entrada amb valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de binari d'entrada amb un valor que no existeix
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de resolucio d'entrada amb un valor que no existeix
ACTIVA_INCORRECTE EXCEPTION; -- excepcio corresponent al codi de binari actiu o no amb un valor incorrecte, diferent de 'S' i 'N'
VALORS_NULL EXCEPTION; -- excepcio corresponent a que tots els valors a modificar sn NULL, s a dir, no hi ha res a modificar

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intCodiBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;  
  
  IF (strHipervincle IS NULL) AND (intMida IS NULL) AND (intResolucio IS NULL) AND (strActiva IS NULL) THEN
    RAISE VALORS_NULL;
  END IF;

  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;
  END IF;

  IF NOT (strActiva IS NULL) THEN
    IF (strActiva <> 'S') AND (strActiva <> 'N') THEN
      RAISE ACTIVA_INCORRECTE;
    END IF;
  END IF;
  
  -- Construim la sentencia SQL d'actualitzaci que s'ha d'executar tenint en compte els parmetres que han estat informats
  strSeparador := '';
  strSQL := 'UPDATE BINARI SET ';

  IF NOT (strHipervincle IS NULL) THEN
    strSQL := strSQL || ' HIPERVINCLE = ''' || strHipervincle || '''';
    strSeparador := ',';
  END IF;

  IF NOT (intMida IS NULL) THEN
    strSQL := strSQL || strSeparador || ' MIDA = ' || intMida ;
    strSeparador := ',';
  END IF;
  
  IF NOT (intResolucio IS NULL) THEN
    strSQL := strSQL || strSeparador || ' RESOLUCIO = ' || intResolucio ;
    strSeparador := ',';
  END IF;
  
  IF NOT (strActiva IS NULL) THEN
    strSQL := strSQL || strSeparador || ' ACTIVA = ''' || strActiva || '''';
  END IF;
  
  strSQL := strSQL || ' WHERE CODI = ' || intCodiBinari;
  -- executem la sentncia SQL que s'acaba de crear
  EXECUTE IMMEDIATE strSQL;
  -- registrem l'execuci del procediment al log
  registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiBinari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap binari amb el codi ' || intCodiBinari;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
  
    WHEN VALORS_NULL THEN
      RSP := 'ERROR: tots els camps sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de la resoluci mnima ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN ACTIVA_INCORRECTE THEN
      RSP := 'ERROR: el codi que indica si l''aplicaci esta activa o no s incorrecte, diferent de ''S'' i ''N''';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaBinariApp','intCodiBinari:' || intCodiBinari || ', strHipervincle:' || strHipervincle 
                || ',intMida:' || intMida || ',intResolucio:' || intResolucio || ',strActiva:' || strActiva,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaBinariApp                                                                      --
-- DESCRIPCIO:             Obtenir totes les dades d'un binari mitjanant un objecte de tipus typBINARI              --
-- PARAMETRES ENTRADA:     intCodiBinari (codi del fitxer binari del qual es volen obtenir les dades)                --
-- PARAMETRES SORTIDA:     typDadesBinari (objecte de tipus typBINARI amb la informaci del binari)                  --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: BINARI_NULL (el codi del binari passat com a parametre d'entrada es NULL)                 --
--                         BINARI_NO_EXISTEIX (el codi del binari passat com a parametre d'entrada no existeix)      --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaBinariApp (intCodiBinari IN BINARI.CODI%TYPE,typDadesBinari OUT typBINARI,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi del binari d'entrada amb valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del binari d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intCodiBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;
  
  strPARAMOUT := '';
  typDadesBinari := typBINARI(intCodiBinari,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a inicialitzar l'objecte amb les dades del binari

  -- obtenim la informaci del fitxer binari i la guardem a la variable typDadesBinari
  SELECT BINARI.CODI,APLICACIO.IDAPLICACIO,SO.NOM,BINARI.VERSIO,BINARI.DATAPUJADA,BINARI.DATABAIXA,BINARI.HIPERVINCLE,BINARI.MIDA,RESOLUCIO.NOM,BINARI.ACTIVA 
  INTO typDadesBinari.codi,typDadesBinari.aplicacio,typDadesBinari.so,typDadesBinari.versio,typDadesBinari.dataPujada,
        typDadesBinari.dataBaixa,typDadesBinari.hipervincle,typDadesBinari.mida,typDadesBinari.resolucio,typDadesBinari.activa
  FROM ((BINARI LEFT JOIN APLICACIO ON BINARI.CODIAPLICACIO=APLICACIO.CODI) LEFT JOIN SO ON (BINARI.CODISO=SO.CODI)) LEFT JOIN RESOLUCIO ON BINARI.RESOLUCIO=RESOLUCIO.CODI
  WHERE BINARI.CODI=intCodiBinari;

  strPARAMOUT := 'codi:' || typDadesBinari.codi || ',' 
                  || 'aplicacio:' || typDadesBinari.aplicacio || ',' 
                  || 'so:' || typDadesBinari.so || ',' 
                  || 'versio:' || typDadesBinari.versio || ',' 
                  || 'dataPujada:' || typDadesBinari.dataPujada || ',' 
                  || 'dataBaixa:' || typDadesBinari.dataBaixa || ',' 
                  || 'hipervincle:' || typDadesBinari.hipervincle || ',' 
                  || 'mida:' || typDadesBinari.mida || ',' 
                  || 'resolucio:' || typDadesBinari.resolucio || ',' 
                  || 'activa:' || typDadesBinari.activa;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiBinari s NULL';
      registreLOG('proConsultaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el binari amb codi ' || intCodiBinari || ' no existeix';
      registreLOG('proConsultaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaBinariApp','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatBinarisApp                                                                         --
-- DESCRIPCIO:             Obtenir un llistat amb les dades de tots els binaris d'una aplicaci                         --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicaci de la qual es volen consultar tots els binaris)        --
-- PARAMETRES SORTIDA:     tblLlistatBinaris (taula de tipus tblLLISTATBINARISAPP amb el resultat de la consulta)       --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)    --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                       --
--                         APLICACIO_NO_EXISTEIX (el parametre d'entrada intCodiAplicacio no correspon a cap aplicaci) --
--------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatBinarisApp (intCodiAplicacio IN APLICACIO.CODI%TYPE,tblLlistatBinaris OUT tblLLISTATBINARISAPP,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regBinari typBINARI := typBINARI(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorBinaris IS -- cursor amb els binaris pertanyents a una aplicaci
  SELECT BINARI.CODI,APLICACIO.IDAPLICACIO,SO.NOM,BINARI.VERSIO,BINARI.DATAPUJADA,BINARI.DATABAIXA,BINARI.HIPERVINCLE,BINARI.MIDA,RESOLUCIO.NOM,BINARI.ACTIVA 
  FROM ((BINARI LEFT JOIN APLICACIO ON BINARI.CODIAPLICACIO=APLICACIO.CODI) LEFT JOIN SO ON (BINARI.CODISO=SO.CODI)) LEFT JOIN RESOLUCIO ON BINARI.RESOLUCIO=RESOLUCIO.CODI
  WHERE BINARI.CODIAPLICACIO=intCodiAplicacio
  ORDER BY BINARI.CODI;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi d'aplicaci d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi d'aplicaci d'entrada amb valor un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;

  tblLlistatBinaris := tblLLISTATBINARISAPP();
  strPARAMOUT := '';

  -- es recorren els binaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorBinaris;
    LOOP 
      FETCH cursorBinaris
        INTO regBinari.codi,regBinari.aplicacio,regBinari.so,regBinari.versio,regBinari.dataPujada,regBinari.dataBaixa,regBinari.hipervincle,regBinari.mida,regBinari.resolucio,regBinari.activa;
        EXIT WHEN cursorBinaris%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'codi:' || regBinari.codi || ',' 
                  || 'aplicacio:' || regBinari.aplicacio || ',' 
                  || 'so:' || regBinari.so || ',' 
                  || 'versio:' || regBinari.versio || ',' 
                  || 'dataPujada:' || regBinari.dataPujada || ',' 
                  || 'dataBaixa:' || regBinari.dataBaixa || ',' 
                  || 'hipervincle:' || regBinari.hipervincle || ',' 
                  || 'mida:' || regBinari.mida || ',' 
                  || 'resolucio:' || regBinari.resolucio || ',' 
                  || 'activa:' || regBinari.activa;
                  
        tblLlistatBinaris.extend; -- s'afegeix un nou registre a l'objecte tblLlistatBinaris
        tblLlistatBinaris(tblLlistatBinaris.count) := regBinari; -- al registre acabat de crear a la linia anterior se li informa el valor
    END LOOP;
	CLOSE cursorBinaris;
  -- registrem l'execuci del procediment al log
  registreLOG('proLlistatBinarisApp','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      registreLOG('proLlistatBinarisApp','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''aplicaci ' || intCodiAplicacio || ' no correspon a cap aplicaci';
      registreLOG('proLlistatBinarisApp','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proLlistatBinarisApp','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
END;
/

------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaDesenvolupador                                                                      --
-- DESCRIPCIO:             Donar d'alta un desenvolupador                                                             --
-- PARAMETRES ENTRADA:     strIdDesenvolupador (identificador del desenvolupador)                                     --
--                         strNomEmpresa (nom de l'empresa desenvolupadora)                                           --
--                         strRepLegal (nom del representant legal)                                                   --
--                         strAdrea (adrea del desenvolupador)                                                      --
--                         strTelefon (telefon del desenvolupador)                                                    --
--                         strPais (pais del desenvolupador)                                                          --
-- PARAMETRES SORTIDA:     intCodiDesenvolupador (codi del desenvolupador acabat de donar d'alta)                     --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)  --
-- EXCEPCIONS CONTROLADES: ID_NULL (el parametre strIdDesenvolupador es NULL)                                         --
--                         ID_EXISTEIX (el parametre strIdDesenvolupador ja existeix)                                 --
--                         PAIS_NULL (el parametre strPais es NULL)                                                   --
--                         PAIS_NO_EXISTEIX (el parametre strPais no existeix)                                        --
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaDesenvolupador (strIdDesenvolupador IN DESENVOLUPADOR.IDDESENVOLUPADOR%TYPE,strNomEmpresa IN DESENVOLUPADOR.NOMEMPRESA%TYPE, strRepLegal IN DESENVOLUPADOR.NOMREPLEGAL%TYPE,
                              strAdrea IN DESENVOLUPADOR.ADREA%TYPE,strTelefon IN DESENVOLUPADOR.TELEFON%TYPE,strPais IN PAIS.CODI%TYPE,intCodiDesenvolupador OUT DESENVOLUPADOR.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
ID_NULL EXCEPTION; -- excepcio corresponent al id del desenvolupador amb valor NULL
ID_EXISTEIX EXCEPTION; -- excepcio corresponent al id del desenvolupador amb un valor que ja existeix
PAIS_NULL EXCEPTION; -- excepcio corresponent al codi del pais amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del pais amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strIdDesenvolupador IS NULL THEN
    RAISE ID_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE IDDESENVOLUPADOR = strIdDesenvolupador;
  IF intComptador > 0 THEN
    RAISE ID_EXISTEIX;
  END IF;  

  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;  

  -- afegim el nou desenvolupador a la plataforma
  INSERT INTO DESENVOLUPADOR (IDDESENVOLUPADOR,NOMEMPRESA,NOMREPLEGAL,ADREA,TELEFON,PAIS) VALUES (strIdDesenvolupador,strNomEmpresa,strRepLegal,strAdrea,strTelefon,strPais);
  SELECT CODI INTO intCodiDesenvolupador FROM DESENVOLUPADOR WHERE IDDESENVOLUPADOR = strIdDesenvolupador; -- recuperem el codi del desenvolupador que s'ha acabat de donar d'alta
  strPARAMOUT := 'intCodiDesenvolupador:' || intCodiDesenvolupador; 
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN ID_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strIdDesenvolupador s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN ID_EXISTEIX THEN
      RSP := 'ERROR: el desenvolupador amb nom identificador ' || strIdDesenvolupador || ' ja existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strPais s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupador','strIdDesenvolupador:' || strIdDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || 
                ', strRepLegal:' || strRepLegal || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon|| ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

----------------------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proBaixaDesenvolupador                                                                                                       --
-- DESCRIPCIO:             Donar de baixa un desenvolupador pel seu codi o b pel seu nom identificador (preval el codi si esta informat)               --
-- PARAMETRES ENTRADA:     intCodiDesenvolupador (codi del desenvolupador que es vol donar de baixa, preval sobre l'altre parametre d'entrada strIdNom) --
--                         strIdNom (nom identificador del desenvolupador, si intCodiDesenvolupador no es NULL aquest parametre es ignorat)             --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                                    --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el parametre intCodiDesenvolupador i strIdNom son NULL)                                                 --
--                         CODI_NO_EXISTEIX (no existeix cap desenvolupador amb el codi igual a intCodiDesenvolupador)                                  --
--                         NOM_NO_EXISTEIX (no existeix cap desenvolupador amb el nom identificador igual a strIdNom)                                   --
--                         DESENVOLUPADOR_DE_BAIXA (el desenvolupador ja estava donat de baixa)                                                         --
----------------------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proBaixaDesenvolupador (intCodiDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,strIdNom IN DESENVOLUPADOR.IDDESENVOLUPADOR%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intCodiDesenvolupadorTemp NUMBER; -- variable per a recollir el codi del desenvolupador quan s'ha informat el nom identificador
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi del desenvolupador d'entrada i nom identificador amb valor NULL
CODI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de desenvolupador d'entrada amb un valor que no existeix
NOM_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al nom identificador del desenvolupador d'entrada amb un valor que no existeix
DESENVOLUPADOR_DE_BAIXA EXCEPTION; -- excepcio corresponent a que el desenvolupador que es vol donar de baixa ja ho esta

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF (intCodiDesenvolupador IS NULL) AND (strIdNom IS NULL) THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  IF NOT (intCodiDesenvolupador IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intCodiDesenvolupador;
    IF intComptador = 0 THEN
      RAISE CODI_NO_EXISTEIX;
    ELSE
      intCodiDesenvolupadorTemp := intCodiDesenvolupador;
    END IF;    
  ELSE 
    SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE IDDESENVOLUPADOR = strIdNom;
    IF intComptador = 0 THEN
      RAISE NOM_NO_EXISTEIX;
    ELSE
      SELECT CODI INTO intCodiDesenvolupadorTemp FROM DESENVOLUPADOR WHERE IDDESENVOLUPADOR = strIdNom;
    END IF;      
  END IF;

  -- comprovem que el desenvolupador no estigui donada de baixa
  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE (CODI=intCodiDesenvolupadorTemp) AND (NOT (DATABAIXA IS NULL));
  IF intComptador > 0 THEN
    RAISE DESENVOLUPADOR_DE_BAIXA;
  END IF;  

  -- per donar de baixa el desenvolupador actualitzem la data de baixa amb la data actual
  UPDATE DESENVOLUPADOR SET DATABAIXA=SYSTIMESTAMP WHERE CODI=intCodiDesenvolupadorTemp;
  -- registrem l'execuci del procediment al log
  registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: els parmetres d''entrada intCodiDesenvolupador i strIdNom sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);

    WHEN CODI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap desenvolupador amb el codi ' || intCodiDesenvolupador;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);

    WHEN NOM_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap desenvolupador amb el nom ' || strIdNom;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_DE_BAIXA THEN
      RSP := 'ERROR: el desenvolupador ja estava donat de baixa';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);
      
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proBaixaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strIdNom:' || strIdNom,strPARAMOUT,RSP,strRTN);
END;
/

--------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proModificaDesenvolupador                                                                          --
-- DESCRIPCIO:             Modificar les dades d'un desenvolupador. Si un parametre de modificaci s NULL s'ignora,          --
--                         no es modifiquen camps amb valors NULL                                                             --
-- PARAMETRES ENTRADA:     intCodiDesenvolupador (codi del desenvolupador del qual es volen modificar les dades)              --
--                         strNomEmpresa (nom de l'empresa desenvolupadora)                                                   --
--                         strRepLegal (nom del representant legal)                                                           --
--                         strAdrea (adrea del desenvolupador)                                                              --
--                         strTelefon (telefon del desenvolupador)                                                            --
--                         strPais (pais del desenvolupador)                                                                  --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)          --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el parametre intCodiDesenvolupador es NULL)                                   --
--                         DESENVOLUPADOR_NO_EXISTEIX (no existeix cap desenvolupador amb codi igual a intCodiDesenvolupador) --
--                         PAIS_NO_EXISTEIX (el parametre strPais no existeix)                                                --
--                         VALORS_NULL (tots els camps a modificar sn NULL)                                                  --
--------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proModificaDesenvolupador (intCodiDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,strNomEmpresa IN DESENVOLUPADOR.NOMEMPRESA%TYPE, strRepLegal IN DESENVOLUPADOR.NOMREPLEGAL%TYPE,
                                                  strAdrea IN DESENVOLUPADOR.ADREA%TYPE,strTelefon IN DESENVOLUPADOR.TELEFON%TYPE,strPais IN PAIS.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strSeparador VARCHAR2 (1 CHAR); -- per afegir el separador dels camps a modificar en la construccio de la sentencia SQL que s'ha d'executar
strSQL VARCHAR2(500 CHAR); -- per construir la sentencia SQL que s'ha d'executar
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi del desenvolupador d'entrada amb valor NULL
DESENVOLUPADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del desenvolupador d'entrada amb un valor que no existeix
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del pais amb un valor que no existeix
VALORS_NULL EXCEPTION; -- excepcio corresponent a que tots els valors a modificar sn NULL, s a dir, no hi ha res a modificar

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiDesenvolupador IS NULL THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intCodiDesenvolupador;
  IF intComptador = 0 THEN
    RAISE DESENVOLUPADOR_NO_EXISTEIX;
  END IF;  
  
  IF (strNomEmpresa IS NULL) AND (strRepLegal IS NULL) AND (strAdrea IS NULL) AND (strTelefon IS NULL)  AND (strPais IS NULL) THEN
    RAISE VALORS_NULL;
  END IF;
  
  IF NOT (strPais IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
    IF intComptador = 0 THEN
      RAISE PAIS_NO_EXISTEIX;
    END IF;
  END IF;

  -- Construim la sentencia SQL d'actualitzaci que s'ha d'executar tenint en compte els parmetres que han estat informats
  strSeparador := '';
  strSQL := 'UPDATE DESENVOLUPADOR SET ';
  
  IF NOT (strNomEmpresa IS NULL) THEN
    strSQL := strSQL || ' NOMEMPRESA = ''' || strNomEmpresa || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strRepLegal IS NULL) THEN
    strSQL := strSQL || strSeparador || ' NOMREPLEGAL = ''' || strRepLegal || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strAdrea IS NULL) THEN
    strSQL := strSQL || strSeparador || ' ADREA = ''' || strAdrea  || '''';
    strSeparador := ',';
  END IF;
  
  IF NOT (strTelefon IS NULL) THEN
    strSQL := strSQL || strSeparador || ' TELEFON = ''' || strTelefon || '''';
  END IF;

  IF NOT (strPais IS NULL) THEN
    strSQL := strSQL || strSeparador || ' PAIS = ''' || strPais || '''';
  END IF;
  
  strSQL := strSQL || ' WHERE CODI = ' || intCodiDesenvolupador;
  -- executem la sentncia SQL que acabem de crear
  EXECUTE IMMEDIATE strSQL;
  -- registrem l'execuci del procediment al log
  registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiDesenvolupador s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap desenvolupador amb el codi ' || intCodiDesenvolupador;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
    WHEN VALORS_NULL THEN
      RSP := 'ERROR: tots els camps sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador || ', strNomEmpresa:' || strNomEmpresa || ', strRepLegal:' || strRepLegal 
                || ',strAdrea:' || strAdrea || ',strTelefon:' || strTelefon || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

----------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaDesenvolupador                                                                            --
-- DESCRIPCIO:             Obtenir totes les dades d'un desenvolupador mitjanant un objecte de tipus typDESENVOLUPADOR         --
-- PARAMETRES ENTRADA:     intCodiDesenvolupador (codi del desenvolupador del qual es volen obtenir les dades)                  --
-- PARAMETRES SORTIDA:     typDadesDesenvolupador (objecte de tipus typDESENVOLUPADOR amb la informaci del desenvolupador)     --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)            --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el codi del desenvolupador passat com a parametre d'entrada es NULL)            --
--                         DESENVOLUPADOR_NO_EXISTEIX (el codi del desenvolupador passat com a parametre d'entrada no existeix) --
----------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaDesenvolupador (intCodiDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,typDadesDesenvolupador OUT typDESENVOLUPADOR,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi del desenvolupador d'entrada amb valor NULL
DESENVOLUPADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del desenvolupador d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiDesenvolupador IS NULL THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intCodiDesenvolupador;
  IF intComptador = 0 THEN
    RAISE DESENVOLUPADOR_NO_EXISTEIX;
  END IF;
  
  strPARAMOUT := '';
  typDadesDesenvolupador := typDESENVOLUPADOR(intCodiDesenvolupador,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a inicialitzar l'objecte amb les dades del desenvolupador
  
  -- obtenim les dades del desenvolupador i les guardem a la variable typDadesDesenvolupador
  SELECT DESENVOLUPADOR.CODI,DESENVOLUPADOR.IDDESENVOLUPADOR,DESENVOLUPADOR.DATAALTA,DESENVOLUPADOR.DATABAIXA,DESENVOLUPADOR.NOMEMPRESA,
          DESENVOLUPADOR.NOMREPLEGAL,DESENVOLUPADOR.ADREA,DESENVOLUPADOR.TELEFON,PAIS.NOM 
  INTO typDadesDesenvolupador.codi,typDadesDesenvolupador.idDesenvolupador,typDadesDesenvolupador.dataAlta,typDadesDesenvolupador.dataBaixa,typDadesDesenvolupador.nomEmpresa,
          typDadesDesenvolupador.nomRepLegal,typDadesDesenvolupador.adrea,typDadesDesenvolupador.telefon,typDadesDesenvolupador.pais
  FROM DESENVOLUPADOR LEFT JOIN PAIS ON DESENVOLUPADOR.PAIS=PAIS.CODI 
  WHERE DESENVOLUPADOR.CODI=intCodiDesenvolupador;

  strPARAMOUT := 'codi:' || typDadesDesenvolupador.codi || ',' 
                  || 'idDesenvolupador:' || typDadesDesenvolupador.idDesenvolupador || ',' 
                  || 'dataAlta:' || typDadesDesenvolupador.dataAlta || ',' 
                  || 'dataBaixa:' || typDadesDesenvolupador.dataBaixa || ',' 
                  || 'nomEmpresa:' || typDadesDesenvolupador.nomEmpresa || ',' 
                  || 'nomRepLegal:' || typDadesDesenvolupador.nomRepLegal || ',' 
                  || 'adrea:' || typDadesDesenvolupador.adrea || ',' 
                  || 'telefon:' || typDadesDesenvolupador.telefon || ',' 
                  || 'pais:' || typDadesDesenvolupador.pais;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiDesenvolupador s NULL';
      registreLOG('proConsultaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el desenvolupador amb codi ' || intCodiDesenvolupador || ' no existeix';
      registreLOG('proConsultaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaDesenvolupador','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaDesenvolupadorBinari                                                               --
-- DESCRIPCIO:             Establir un desenvolupador d'un fitxer binari                                             --
-- PARAMETRES ENTRADA:     intDesenvolupador (codi del desenvolupador que es vol vincular a un fitxer binari)        --
--                         intBinari (codi del fitxer binari que es vol vincular al desenvolupador)                  --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el parametre intDesenvolupador es NULL)                              --
--                         DESENVOLUPADOR_NO_EXISTEIX (el codi del desenvolupador no existeix)                       --
--                         BINARI_NULL (el parametre intBinari es NULL)                                              --
--                         BINARI_NO_EXISTEIX (el codi del binari no existeix)                                       --
--                         VINCULACIO_EXISTEIX (la vinculacio entre el desenvolupador i el binari ja existeix)       --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaDesenvolupadorBinari (intDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,intBinari IN BINARI.CODI%TYPE, RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi del desenvolupador amb valor NULL
DESENVOLUPADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del desenvolupador amb un valor que no existeix
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi del binari amb un valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del binari amb un valor que no existeix
VINCULACIO_EXISTEIX EXCEPTION; -- excepcio corresponent a que la vinculacio entre el desenvolupador i el binari ja existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intDesenvolupador IS NULL THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intDesenvolupador;
  IF intComptador = 0 THEN
    RAISE DESENVOLUPADOR_NO_EXISTEIX;
  END IF;  

  IF intBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADORBINARI WHERE (CODIDESENVOLUPADOR = intDesenvolupador) AND (CODIBINARI = intBinari);
  IF intComptador > 0 THEN
    RAISE VINCULACIO_EXISTEIX;
  END IF;  

  -- inserim la vinculaci entre el desenvolupador i el fitxer binari a la plataforma
  INSERT INTO DESENVOLUPADORBINARI (CODIDESENVOLUPADOR,CODIBINARI) VALUES (intDesenvolupador,intBinari);
  -- registrem l'execuci del procediment al log    
  registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intDesenvolupador s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del desenvolupador ' || intDesenvolupador || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intBinari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del binari ' || intBinari || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN VINCULACIO_EXISTEIX THEN
      RSP := 'ERROR: la vinculaci entre el desenvolupador ' || intDesenvolupador || ' i el binari ' || intBinari || ' ja existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proBaixaDesenvolupadorBinari                                                              --
-- DESCRIPCIO:             Desvincular un desenvolupador d'un fitxer binari                                          --
-- PARAMETRES ENTRADA:     intDesenvolupador (codi del desenvolupador que es vol desvincular a un fitxer binari)     --
--                         intBinari (codi del fitxer binari que es vol desvincular al desenvolupador)               --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el parametre intDesenvolupador es NULL)                              --
--                         DESENVOLUPADOR_NO_EXISTEIX (el codi del desenvolupador no existeix)                       --
--                         BINARI_NULL (el parametre intBinari es NULL)                                              --
--                         BINARI_NO_EXISTEIX (el codi del binari no existeix)                                       --
--                         VINCULACIO_NO_EXISTEIX (la vinculacio entre el desenvolupador i el binari no existeix)    --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proBaixaDesenvolupadorBinari (intDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,intBinari IN BINARI.CODI%TYPE, RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi del desenvolupador amb valor NULL
DESENVOLUPADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del desenvolupador amb un valor que no existeix
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi del binari amb un valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del binari amb un valor que no existeix
VINCULACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a que la vinculacio entre el desenvolupador i el binari no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intDesenvolupador IS NULL THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intDesenvolupador;
  IF intComptador = 0 THEN
    RAISE DESENVOLUPADOR_NO_EXISTEIX;
  END IF;  

  IF intBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADORBINARI WHERE (CODIDESENVOLUPADOR = intDesenvolupador) AND (CODIBINARI = intBinari);
  IF intComptador = 0 THEN
    RAISE VINCULACIO_NO_EXISTEIX;
  END IF;  

  -- esborrem la vinculaci entre el desenvolupador i el fitxer binari de la plataforma
  DELETE FROM DESENVOLUPADORBINARI WHERE (CODIDESENVOLUPADOR = intDesenvolupador) AND (CODIBINARI = intBinari);
  -- registrem l'execuci del procediment al log    
  registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intDesenvolupador s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del desenvolupador ' || intDesenvolupador || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intBinari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del binari ' || intBinari || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN VINCULACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: la vinculaci entre el desenvolupador ' || intDesenvolupador || ' i el binari ' || intBinari || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaDesenvolupadorBinari','intDesenvolupador:' || intDesenvolupador || ', intBinari:' || intBinari,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaBinarisDes                                                                                       --
-- DESCRIPCIO:             Obtenir un llistat amb tots els fitxers binaris que ha creat un desenvolupador                              --
-- PARAMETRES ENTRADA:     intCodiDesenvolupador (codi del desenvolupador del qual es volen consultar tots els binaris)                --
-- PARAMETRES SORTIDA:     tblLlistatBinaris (taula de tipus tblLLISTATBINARISAPP amb el resultat de la consulta)                      --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                   --
-- EXCEPCIONS CONTROLADES: DESENVOLUPADOR_NULL (el parametre intCodiDesenvolupador es NULL)                                            --
--                         DESENVOLUPADOR_NO_EXISTEIX (el parametre d'entrada intCodiDesenvolupador no correspon a cap desenvolupador) --
-----------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaBinarisDes (intCodiDesenvolupador IN DESENVOLUPADOR.CODI%TYPE,tblLlistatBinaris OUT tblLLISTATBINARISAPP,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regBinari typBINARI := typBINARI(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorBinaris IS -- cursor amb els binaris que ha realitzat un desenvolupador
  SELECT BINARI.CODI,APLICACIO.IDAPLICACIO,SO.NOM,BINARI.VERSIO,BINARI.DATAPUJADA,BINARI.DATABAIXA,BINARI.HIPERVINCLE,BINARI.MIDA,RESOLUCIO.NOM,BINARI.ACTIVA 
  FROM ((BINARI LEFT JOIN APLICACIO ON BINARI.CODIAPLICACIO=APLICACIO.CODI) LEFT JOIN SO ON (BINARI.CODISO=SO.CODI)) LEFT JOIN RESOLUCIO ON BINARI.RESOLUCIO=RESOLUCIO.CODI
  WHERE BINARI.CODI IN (SELECT CODIBINARI FROM DESENVOLUPADORBINARI WHERE CODIDESENVOLUPADOR=intCodiDesenvolupador)
  ORDER BY BINARI.CODI;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DESENVOLUPADOR_NULL EXCEPTION; -- excepcio corresponent al codi de desenvolupador d'entrada amb valor NULL
DESENVOLUPADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de desenvolupador d'entrada amb valor un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiDesenvolupador IS NULL THEN
    RAISE DESENVOLUPADOR_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DESENVOLUPADOR WHERE CODI = intCodiDesenvolupador;
  IF intComptador = 0 THEN
    RAISE DESENVOLUPADOR_NO_EXISTEIX;
  END IF;

  tblLlistatBinaris := tblLLISTATBINARISAPP();
  strPARAMOUT := '';

  -- es recorren els binaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorBinaris;
    LOOP 
      FETCH cursorBinaris
        INTO regBinari.codi,regBinari.aplicacio,regBinari.so,regBinari.versio,regBinari.dataPujada,regBinari.dataBaixa,regBinari.hipervincle,regBinari.mida,regBinari.resolucio,regBinari.activa;
        EXIT WHEN cursorBinaris%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'codi:' || regBinari.codi || ',' 
                  || 'aplicacio:' || regBinari.aplicacio || ',' 
                  || 'so:' || regBinari.so || ',' 
                  || 'versio:' || regBinari.versio || ',' 
                  || 'dataPujada:' || regBinari.dataPujada || ',' 
                  || 'dataBaixa:' || regBinari.dataBaixa || ',' 
                  || 'hipervincle:' || regBinari.hipervincle || ',' 
                  || 'mida:' || regBinari.mida || ',' 
                  || 'resolucio:' || regBinari.resolucio || ',' 
                  || 'activa:' || regBinari.activa;
                  
        tblLlistatBinaris.extend; -- afegim un nou registre a l'objecte tblLlistatBinaris
        tblLlistatBinaris(tblLlistatBinaris.count) := regBinari; -- li donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorBinaris;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaBinarisDes','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excecpions
  EXCEPTION
    WHEN DESENVOLUPADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiDesenvolupador s NULL';
      registreLOG('proConsultaBinarisDes','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);

    WHEN DESENVOLUPADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de desenvolupador ' || intCodiDesenvolupador || ' no existeix';
      registreLOG('proConsultaBinarisDes','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaBinarisDes','intCodiDesenvolupador:' || intCodiDesenvolupador,strPARAMOUT,RSP,strRTN);
END;
/

---------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaDesBinari                                                                                --
-- DESCRIPCIO:             Obtenir un llistat amb tots desenvolupadors que han implementat un binari                           --
-- PARAMETRES ENTRADA:     intCodiBinari (codi del binari del qual es volen consultar tots els desenvolupadors)                --
-- PARAMETRES SORTIDA:     tblDesenvolupadors (taula de tipus tblLLISTATDESENVOLUPADORS amb el resultat de la consulta)        --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)           --
-- EXCEPCIONS CONTROLADES: BINARI_NULL (el parametre intCodiBinari es NULL)                                                    --
--                         BINARI_NO_EXISTEIX (el parametre d'entrada intCodiBinari no correspon a cap binari)                 --
---------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaDesBinari (intCodiBinari IN BINARI.CODI%TYPE,tblDesenvolupadors OUT tblLLISTATDESENVOLUPADORS,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regDesenvolupador typDESENVOLUPADOR := typDESENVOLUPADOR(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorDesenvolupadors IS -- cursor amb els desenvolupadors que han implementat un binari
  SELECT DESENVOLUPADOR.CODI,DESENVOLUPADOR.IDDESENVOLUPADOR,DESENVOLUPADOR.DATAALTA,DESENVOLUPADOR.DATABAIXA,DESENVOLUPADOR.NOMEMPRESA,DESENVOLUPADOR.NOMREPLEGAL,DESENVOLUPADOR.ADREA,DESENVOLUPADOR.TELEFON,PAIS.NOM 
  FROM DESENVOLUPADOR LEFT JOIN PAIS ON DESENVOLUPADOR.PAIS=PAIS.CODI 
  WHERE DESENVOLUPADOR.CODI IN (SELECT CODIDESENVOLUPADOR FROM DESENVOLUPADORBINARI WHERE CODIBINARI=intCodiBinari) 
  ORDER BY DESENVOLUPADOR.CODI;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
BINARI_NULL EXCEPTION; -- excepcio corresponent al codi de binari d'entrada amb valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de binari d'entrada amb valor un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intCodiBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;

  tblDesenvolupadors := tblLLISTATDESENVOLUPADORS();
  strPARAMOUT := '';

  -- es recorren els binaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDesenvolupadors;
    LOOP 
      FETCH cursorDesenvolupadors
        INTO regDesenvolupador.codi,regDesenvolupador.idDesenvolupador,regDesenvolupador.dataAlta,regDesenvolupador.dataBaixa,regDesenvolupador.nomEmpresa,
              regDesenvolupador.nomRepLegal,regDesenvolupador.adrea,regDesenvolupador.telefon,regDesenvolupador.pais;
        EXIT WHEN cursorDesenvolupadors%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;

        strPARAMOUT := strPARAMOUT || 'codi:' || regDesenvolupador.codi || ',' 
                  || 'idDesenvolupador:' || regDesenvolupador.idDesenvolupador || ',' 
                  || 'dataAlta:' || regDesenvolupador.dataAlta || ',' 
                  || 'dataBaixa:' || regDesenvolupador.dataBaixa || ',' 
                  || 'nomEmpresa:' || regDesenvolupador.nomEmpresa || ',' 
                  || 'nomRepLegal:' || regDesenvolupador.nomRepLegal || ',' 
                  || 'adrea:' || regDesenvolupador.adrea || ',' 
                  || 'telefon:' || regDesenvolupador.telefon || ',' 
                  || 'pais:' || regDesenvolupador.pais;
                  
        tblDesenvolupadors.extend; -- afegim un nou registre a l'objecte tblDesenvolupadors
        tblDesenvolupadors(tblDesenvolupadors.count) := regDesenvolupador; -- donem valor al registre que acabem de crear a la linia anterior
    END LOOP;
	CLOSE cursorDesenvolupadors;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaDesBinari','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiBinari s NULL';
      registreLOG('proConsultaDesBinari','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de binari ' || intCodiBinari || ' no existeix';
      registreLOG('proConsultaDesBinari','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaDesBinari','intCodiBinari:' || intCodiBinari,strPARAMOUT,RSP,strRTN);
END;
/

---------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaDesAplicacio                                                                       --
-- DESCRIPCIO:             Obtenir un llistat amb tots desenvolupadors que han implementat una aplicacio                 --
-- PARAMETRES ENTRADA:     intCodiAplicacio (codi de l'aplicaci de la qual es volen consultar tots els desenvolupadors) --
-- PARAMETRES SORTIDA:     tblDesenvolupadors (taula de tipus tblLLISTATDESENVOLUPADORS amb el resultat de la consulta)  --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)     --
-- EXCEPCIONS CONTROLADES: APLICACIO_NULL (el parametre intCodiAplicacio es NULL)                                        --
--                         APLICACIO_NO_EXISTEIX (el parametre d'entrada intCodiAplicacio no correspon a cap binari)     --
---------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaDesAplicacio (intCodiAplicacio IN APLICACIO.CODI%TYPE,tblDesenvolupadors OUT tblLLISTATDESENVOLUPADORS,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regDesenvolupador typDESENVOLUPADOR := typDESENVOLUPADOR(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorDesenvolupadors IS -- cursor amb els desenvolupadors que han implementat un binari
  SELECT DESENVOLUPADOR.CODI,DESENVOLUPADOR.IDDESENVOLUPADOR,DESENVOLUPADOR.DATAALTA,DESENVOLUPADOR.DATABAIXA,DESENVOLUPADOR.NOMEMPRESA,DESENVOLUPADOR.NOMREPLEGAL,DESENVOLUPADOR.ADREA,DESENVOLUPADOR.TELEFON,PAIS.NOM 
  FROM DESENVOLUPADOR LEFT JOIN PAIS ON DESENVOLUPADOR.PAIS=PAIS.CODI 
  WHERE DESENVOLUPADOR.CODI IN (SELECT DISTINCT(CODIDESENVOLUPADOR) FROM DESENVOLUPADORBINARI WHERE CODIBINARI IN (SELECT CODI FROM BINARI WHERE CODIAPLICACIO=intCodiAplicacio)) 
  ORDER BY DESENVOLUPADOR.CODI;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APLICACIO_NULL EXCEPTION; -- excepcio corresponent al codi de l'aplicacio d'entrada amb valor NULL
APLICACIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'aplicacio d'entrada amb valor un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiAplicacio IS NULL THEN
    RAISE APLICACIO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = intCodiAplicacio;
  IF intComptador = 0 THEN
    RAISE APLICACIO_NO_EXISTEIX;
  END IF;

  tblDesenvolupadors := tblLLISTATDESENVOLUPADORS();
  strPARAMOUT := '';

  -- es recorren els binaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDesenvolupadors;
    LOOP 
      FETCH cursorDesenvolupadors
        INTO regDesenvolupador.codi,regDesenvolupador.idDesenvolupador,regDesenvolupador.dataAlta,regDesenvolupador.dataBaixa,regDesenvolupador.nomEmpresa,
              regDesenvolupador.nomRepLegal,regDesenvolupador.adrea,regDesenvolupador.telefon,regDesenvolupador.pais;
        EXIT WHEN cursorDesenvolupadors%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;

        strPARAMOUT := strPARAMOUT || 'codi:' || regDesenvolupador.codi || ',' 
                  || 'idDesenvolupador:' || regDesenvolupador.idDesenvolupador || ',' 
                  || 'dataAlta:' || regDesenvolupador.dataAlta || ',' 
                  || 'dataBaixa:' || regDesenvolupador.dataBaixa || ',' 
                  || 'nomEmpresa:' || regDesenvolupador.nomEmpresa || ',' 
                  || 'nomRepLegal:' || regDesenvolupador.nomRepLegal || ',' 
                  || 'adrea:' || regDesenvolupador.adrea || ',' 
                  || 'telefon:' || regDesenvolupador.telefon || ',' 
                  || 'pais:' || regDesenvolupador.pais;
                  
        tblDesenvolupadors.extend; -- afegim un nou registre a l'objecte tblDesenvolupadors
        tblDesenvolupadors(tblDesenvolupadors.count) := regDesenvolupador; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorDesenvolupadors;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaDesAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APLICACIO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiAplicacio s NULL';
      registreLOG('proConsultaDesAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN APLICACIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de l''aplicaci ' || intCodiAplicacio || ' no existeix';
      registreLOG('proConsultaDesAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaDesAplicacio','intCodiAplicacio:' || intCodiAplicacio,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaDispUsuari                                                                     --
-- DESCRIPCIO:             Obtenir un llistat amb tots els dispositius d'un usuari                                   --
-- PARAMETRES ENTRADA:     intCodiUsuari (codi de l'usuari del qual es volen consultar tots els seus dispositius)    --
-- PARAMETRES SORTIDA:     tblDispositius (taula de tipus tblLLISTATDISPOSITIUS amb el resultat de la consulta)      --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: USUARI_NULL (el parametre intCodiUsuari es NULL)                                          --
--                         USUARI_NO_EXISTEIX (el parametre d'entrada intCodiUsuari no correspon a cap usuari)       --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaDispUsuari (intCodiUsuari IN USUARI.CODI%TYPE,tblDispositius OUT tblLLISTATDISPOSITIUS,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regDispositiu typDISPOSITIU := typDISPOSITIU(NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorDispositius IS -- cursor amb els dispositius d'un usuari
  SELECT DISPOSITIU.CODI,DISPOSITIU.IMEI,MODELDISPOSITIU.NOM,RESOLUCIO.NOM,SO.NOM 
  FROM ((DISPOSITIU LEFT JOIN MODELDISPOSITIU ON DISPOSITIU.MODELDISP=MODELDISPOSITIU.CODI) LEFT JOIN RESOLUCIO ON DISPOSITIU.RESOLUCIO=RESOLUCIO.CODI) LEFT JOIN SO ON DISPOSITIU.SO=SO.CODI
  WHERE DISPOSITIU.USUARI = intCodiUsuari
  ORDER BY DISPOSITIU.CODI;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USUARI_NULL EXCEPTION; -- excepcio corresponent al codi de l'usuari d'entrada amb valor NULL
USUARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'usuari d'entrada amb valor un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiUsuari IS NULL THEN
    RAISE USUARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE CODI = intCodiUsuari;
  IF intComptador = 0 THEN
    RAISE USUARI_NO_EXISTEIX;
  END IF;

  tblDispositius := tblLLISTATDISPOSITIUS();
  strPARAMOUT := '';

  -- es recorren els binaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDispositius;
    LOOP 
      FETCH cursorDispositius
        INTO regDispositiu.codi,regDispositiu.IMEI,regDispositiu.modelDisp,regDispositiu.resolucio,regDispositiu.so;
        EXIT WHEN cursorDispositius%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;

        strPARAMOUT := strPARAMOUT || 'codi:' || regDispositiu.codi || ',' 
                  || 'IMEI:' || regDispositiu.IMEI || ',' 
                  || 'modelDisp:' || regDispositiu.modelDisp || ',' 
                  || 'resolucio:' || regDispositiu.resolucio || ',' 
                  || 'so:' || regDispositiu.so;
                  
        tblDispositius.extend; -- afegim un nou registre a l'objecte tblDispositius
        tblDispositius(tblDispositius.count) := regDispositiu; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorDispositius;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaDispUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN USUARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiUsuari s NULL';
      registreLOG('proConsultaDispUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);

    WHEN USUARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''usuari ' || intCodiUsuari || ' no existeix';
      registreLOG('proConsultaDispUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaDispUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);
END;
/

------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaUsuari                                                                              --
-- DESCRIPCIO:             Donar d'alta un usuari                                                                     --
-- PARAMETRES ENTRADA:     strMobil (numero de mobil de l'usuari)                                                     --
--                         strOperador (codi de l'operador de l'usuari)                                               --
--                         strEmail (adrea e-mail de l'usuari)                                                       --
--                         strPais (codi ISO 3166-1 alfa-2 del pais de l'usuari)                                      --
-- PARAMETRES SORTIDA:     intCodiUsuari (codi de l'usuari acabat de donar d'alta)                                    --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)  --
-- EXCEPCIONS CONTROLADES: MOBIL_NULL (el parametre strMobil es NULL)                                                 --
--                         MOBIL_EXISTEIX (el parametre strMobil ja existeix)                                         --
--                         OPERADOR_NULL (el parametre strOperador es NULL)                                           --
--                         OPERADOR_NO_EXISTEIX (el parametre strOperador no existeix)                                --
--                         PAIS_NULL (el parametre strPais es NULL)                                                   --
--                         PAIS_NO_EXISTEIX (el parametre strPais no existeix)                                        --
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaUsuari (strMobil IN USUARI.MOBIL%TYPE,strOperador IN OPERADOR.CODI%TYPE, strEmail IN USUARI.EMAIL%TYPE,strPais IN PAIS.CODI%TYPE,intCodiUsuari OUT USUARI.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
MOBIL_NULL EXCEPTION; -- excepcio corresponent al numero de mobil d'entrada amb valor NULL
MOBIL_EXISTEIX EXCEPTION; -- excepcio corresponent al numero de mobil d'entrada amb un valor que ja existeix
OPERADOR_NULL EXCEPTION; -- excepcio corresponent a l'operador d'entrada amb valor NULL
OPERADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a l'operador d'entrada amb un valor que no existeix
PAIS_NULL EXCEPTION; -- excepcio corresponent al codi del pais d'entrada amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de pais d'entrada amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strMobil IS NULL THEN
    RAISE MOBIL_NULL;
  END IF;

  IF strOperador IS NULL THEN
    RAISE OPERADOR_NULL;
  END IF;

  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;
  
  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE MOBIL = strMobil;
  IF intComptador > 0 THEN
    RAISE MOBIL_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM OPERADOR WHERE CODI = strOperador;
  IF intComptador = 0 THEN
    RAISE OPERADOR_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;  

  -- afegim el nou usuari a la plataforma
  INSERT INTO USUARI (MOBIL,OPERADOR,EMAIL,PAIS) VALUES (strMobil,strOperador,strEmail,strPais);
  SELECT CODI INTO intCodiUsuari FROM USUARI WHERE MOBIL = strMobil; -- recuperem el codi de l'usuari acabat de donar d'alta
  strPARAMOUT := 'intCodiUsuari:' || intCodiUsuari; 
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN MOBIL_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strMobil s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OPERADOR_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strOperador s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strPais s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN MOBIL_EXISTEIX THEN
      RSP := 'ERROR: el nmero de mobil ja existeix per a un altre usuari';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OPERADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''operador ' || strOperador || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proAltaUsuari','strMobil:' || strMobil || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

-------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proBaixaUsuari                                                                                                --
-- DESCRIPCIO:             Donar de baixa un usuari pel seu codi o b pel seu numero de mobil (preval el codi si esta informat)          --
-- PARAMETRES ENTRADA:     intCodiUsuari (codi de l'usuari que es vol donar de baixa, preval sobre l'altre parametre d'entrada strMobil) --
--                         strMobil (numero de mobil de l'usuari, si intCodiUsuari no es NULL aquest parametre es ignorat)               --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                     --
-- EXCEPCIONS CONTROLADES: USUARI_NULL (el parametre intCodiUsuari i strMobil son NULL)                                                  --
--                         CODI_NO_EXISTEIX (no existeix cap usuari amb el codi igual a intCodiUsuari)                                   --
--                         MOBIL_NO_EXISTEIX (no existeix cap usuari amb el mobil igual a strMobil)                                      --
--                         USUARI_DE_BAIXA (l'usuari ja estava donat de baixa)                                                           --
-------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proBaixaUsuari (intCodiUsuari IN USUARI.CODI%TYPE,strMobil IN USUARI.MOBIL%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intCodiUsuariTemp NUMBER; -- variable per a recollir el codi d'usuari quan s'ha informat el numero de mobil
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USUARI_NULL EXCEPTION; -- excepcio corresponent al codi d'usuari d'entrada i mobil amb valor NULL
CODI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi d'usuari d'entrada amb un valor que no existeix
MOBIL_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al mobil d'entrada amb un valor que no existeix
USUARI_DE_BAIXA EXCEPTION; -- excepcio corresponent a que l'usuari ja estava donat de baixa

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF (intCodiUsuari IS NULL) AND (strMobil IS NULL) THEN
    RAISE USUARI_NULL;
  END IF;

  IF NOT (intCodiUsuari IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM USUARI WHERE CODI = intCodiUsuari;
    IF intComptador = 0 THEN
      RAISE CODI_NO_EXISTEIX;
    ELSE
      intCodiUsuariTemp := intCodiUsuari;
    END IF;    
  ELSE 
    SELECT COUNT(*) INTO intComptador FROM USUARI WHERE MOBIL = strMobil;
    IF intComptador = 0 THEN
      RAISE MOBIL_NO_EXISTEIX;
    ELSE
      SELECT CODI INTO intCodiUsuariTemp FROM USUARI WHERE MOBIL = strMobil;
    END IF;      
  END IF;

  -- comprovem que l'usuari no estigui donat de baixa
  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE (CODI=intCodiUsuariTemp) AND (NOT (DATABAIXA IS NULL));
  IF intComptador > 0 THEN
    RAISE USUARI_DE_BAIXA;
  END IF;  

  -- per donar de baixa l'usuari actualitzem el camp data de baixa amb la data actual
  UPDATE USUARI SET DATABAIXA=SYSTIMESTAMP WHERE CODI=intCodiUsuariTemp;
  -- registrem l'execuci del procediment al log
  registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN USUARI_NULL THEN
      RSP := 'ERROR: els parmetres d''entrada intCodiUsuari i strMobil sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);

    WHEN CODI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap usuari amb el codi ' || intCodiUsuari;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);

    WHEN MOBIL_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap usuari amb el nmero de mobil ' || strMobil;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);

    WHEN USUARI_DE_BAIXA THEN
      RSP := 'ERROR: l''usuari ja estava donat de baixa';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proBaixaUsuari','intCodiUsuari:' || intCodiUsuari || ', strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proModificaUsuari                                                                         --
-- DESCRIPCIO:             Modificar les dades d'un usuari. Si un parametre de modificaci s NULL s'ignora,         --
--                         no es modifiquen camps amb valors NULL                                                    --
-- PARAMETRES ENTRADA:     intCodiUsuari (codi de l'usuari del qual es volen modificar les dades)                    --
--                         strOperador (nou codi de l'operador de l'usuari)                                          --
--                         strEmail (nova adrea e-mail de l'usuari)                                                 --
--                         strPais (nou codi ISO 3166-1 alfa-2 del pais de l'usuari)                                 --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: USUARI_NULL (el parametre intCodiUsuari es NULL)                                          --
--                         USUARI_NO_EXISTEIX (no existeix cap usuari amb codi igual a intCodiUsuari)                --
--                         OPERADOR_NO_EXISTEIX (el parametre strOperador no existeix)                               --
--                         PAIS_NO_EXISTEIX (el parametre strPais no existeix)                                       --
--                         VALORS_NULL (tots els camps a modificar sn NULL)                                         --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proModificaUsuari (intCodiUsuari IN USUARI.CODI%TYPE,strOperador IN OPERADOR.CODI%TYPE, strEmail IN USUARI.EMAIL%TYPE,strPais IN PAIS.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strSeparador VARCHAR2 (1 CHAR); -- per afegir el separador dels camps a modificar en la construccio de la sentencia SQL que s'ha d'executar
strSQL VARCHAR2(500 CHAR); -- per construir la sentencia SQL que s'ha d'executar
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USUARI_NULL EXCEPTION; -- excepcio corresponent a l'usuari d'entrada amb valor NULL
USUARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a l'usuari d'entrada amb un valor que no existeix
OPERADOR_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a l'operador d'entrada amb un valor que no existeix
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de pais d'entrada amb un valor que no existeix
VALORS_NULL EXCEPTION; -- excepcio corresponent a que tots els valors a modificar sn NULL, s a dir, no hi ha res a modificar

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiUsuari IS NULL THEN
    RAISE USUARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE CODI = intCodiUsuari;
  IF intComptador = 0 THEN
    RAISE USUARI_NO_EXISTEIX;
  END IF;  
  
  IF (strOperador IS NULL) AND (strEmail IS NULL) AND (strPais IS NULL) THEN
    RAISE VALORS_NULL;
  END IF;
  
  IF NOT (strOperador IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM OPERADOR WHERE CODI = strOperador;
    IF intComptador = 0 THEN
      RAISE OPERADOR_NO_EXISTEIX;
    END IF;
  END IF;

  IF NOT (strPais IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
    IF intComptador = 0 THEN
      RAISE PAIS_NO_EXISTEIX;
    END IF;
  END IF;
  
  -- Construim la sentencia SQL d'actualitzaci que s'ha d'executar tenint en compte els parmetres que estan informats
  strSeparador := '';
  strSQL := 'UPDATE USUARI SET ';
  
  IF NOT (strOperador IS NULL) THEN
    strSQL := strSQL || ' OPERADOR = ''' || strOperador || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strEmail IS NULL) THEN
    strSQL := strSQL || strSeparador || ' EMAIL = ''' || strEmail || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strPais IS NULL) THEN
    strSQL := strSQL || strSeparador || ' PAIS = ''' || strPais || '''';
  END IF;
  
  strSQL := strSQL || ' WHERE CODI = ' || intCodiUsuari;
  -- executem la sentncia SQL que s'ha format
  EXECUTE IMMEDIATE strSQL;
  -- registrem l'execuci del procediment al log
  registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN USUARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiUsuari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN USUARI_NO_EXISTEIX THEN
      RSP := 'ERROR: no existeix cap usuari amb el codi ' || intCodiUsuari;
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
    WHEN VALORS_NULL THEN
      RSP := 'ERROR: tots els camps sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OPERADOR_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi d''operador ' || strOperador || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de pais ' || strPais || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
     registreLOG('proModificaUsuari','intCodiUsuari:' || intCodiUsuari || ', strOperador:' || strOperador || ', strEmail:' || strEmail || ',strPais:' || strPais,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proConsultaUsuari                                                                         --
-- DESCRIPCIO:             Obtenir totes les dades d'un usuari mitjanant un objecte de tipus typUSUARI              --
-- PARAMETRES ENTRADA:     intCodiUsuari (codi de l'usuari del qual es volen obtenir les dades)                      --
-- PARAMETRES SORTIDA:     typDadesUsuari (objecte de tipus typUSUARI amb la informaci de l'usuari)                 --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: USUARI_NULL (el codi de l'usuari passat com a parametre d'entrada es NULL)                --
--                         USUARI_NO_EXISTEIX (el codi de l'usuari passat com a parametre d'entrada no existeix)     --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proConsultaUsuari (intCodiUsuari IN USUARI.CODI%TYPE,typDadesUsuari OUT typUSUARI,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USUARI_NULL EXCEPTION; -- excepcio corresponent al codi de l'usuari d'entrada amb valor NULL
USUARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'usuari d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiUsuari IS NULL THEN
    RAISE USUARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE CODI = intCodiUsuari;
  IF intComptador = 0 THEN
    RAISE USUARI_NO_EXISTEIX;
  END IF;
  
  strPARAMOUT := '';
  typDadesUsuari := typUSUARI(intCodiUsuari,NULL,NULL,NULL,NULL,NULL,NULL); -- per a inicialitzar l'objecte amb les dades de l'usuari
  -- obtenim les dades de l'usuari i les guardem a la variable typDadesUsuari
  SELECT USUARI.CODI,USUARI.DATAALTA,USUARI.DATABAIXA,USUARI.MOBIL,OPERADOR.NOM,USUARI.EMAIL,PAIS.NOM
  INTO typDadesUsuari.codi,typDadesUsuari.dataAlta,typDadesUsuari.dataBaixa,typDadesUsuari.mobil,typDadesUsuari.operador,typDadesUsuari.email,typDadesUsuari.pais
  FROM (USUARI LEFT JOIN OPERADOR ON (USUARI.OPERADOR=OPERADOR.CODI)) LEFT JOIN PAIS ON (USUARI.PAIS=PAIS.CODI)
  WHERE USUARI.CODI=intCodiUsuari;
   
  strPARAMOUT := 'codi:' || typDadesUsuari.codi || ',' 
                  || 'dataAlta:' || typDadesUsuari.dataAlta || ',' 
                  || 'dataBaixa:' || typDadesUsuari.dataBaixa || ',' 
                  || 'mobil:' || typDadesUsuari.mobil || ',' 
                  || 'operador:' || typDadesUsuari.operador || ',' 
                  || 'email:' || typDadesUsuari.email || ',' 
                  || 'pais:' || typDadesUsuari.pais;
  -- registrem l'execuci del procediment al log
  registreLOG('proConsultaUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN USUARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiUsuari s NULL';
      registreLOG('proConsultaUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);

    WHEN USUARI_NO_EXISTEIX THEN
      RSP := 'ERROR: l''usuari amb codi ' || intCodiUsuari || ' no existeix';
      registreLOG('proConsultaUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proConsultaUsuari','intCodiUsuari:' || intCodiUsuari,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proAltaDispositiuUsuari                                                                   --
-- DESCRIPCIO:             Donar d'alta un dispositiu per a un usuari                                                --
-- PARAMETRES ENTRADA:     intCodiUsuari (codi d'usuari al qual es vol donar d'alta un nou dispositiu)               --
--                         strIMEI (numero IMEI del nou dispositiu de l'usuari)                                      --
--                         strModelDispositiu (codi del model del dispositiu)                                        --
--                         intResolucio (codi de la resolucio del dispositiu)                                        --
--                         strSO (codi del sistema operatiu del dispositiu)                                          --
-- PARAMETRES SORTIDA:     intCodiDispositiu (codi del dispositiu acabat de donar d'alta)                            --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: USUARI_NULL (el parametre intCodiUsuari es NULL)                                          --
--                         USUARI_NO_EXISTEIX (el codi d'usuari no existeix)                                         --
--                         IMEI_EXISTEIX (el parametre strIMEI ja existeix)                                          --
--                         MODEL_NULL (el parametre strModelDispositiu es NULL)                                      --
--                         MODEL_NO_EXISTEIX (el codi del model del dispositiu no existeix)                          --
--                         RESOLUCIO_NO_EXISTEIX (el codi de la resolucio del dispositiu no existeix)                --
--                         SO_NULL (el parametre strSO es NULL)                                                      --
--                         SO_NO_EXISTEIX (el codi del sistema operatiu del dispositiu no existeix)                  --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proAltaDispositiuUsuari (intCodiUsuari IN USUARI.CODI%TYPE,strIMEI IN DISPOSITIU.IMEI%TYPE, strModelDispositiu IN DISPOSITIU.MODELDISP%TYPE,
                            intResolucio IN RESOLUCIO.CODI%TYPE,strSO IN SO.CODI%TYPE,intCodiDispositiu OUT DISPOSITIU.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USUARI_NULL EXCEPTION; -- excepcio corresponent al codi d'usuari amb valor NULL
USUARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de l'usuari amb un valor que no existeix
IMEI_EXISTEIX EXCEPTION; -- excepcio corresponent al codi IMEI del dispositiu amb un valor que ja existeix
MODEL_NULL EXCEPTION; -- excepcio corresponent al codi del model del dispositiu amb valor NULL
MODEL_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del model del dispositiu amb un valor que no existeix
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de la resolucio del dispositiu amb un valor que no existeix
SO_NULL EXCEPTION; -- excepcio corresponent al codi del sistema operatiu del dispositiu amb valor NULL
SO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del sistema operatiu del dispositiu amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiUsuari IS NULL THEN
    RAISE USUARI_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE CODI = intCodiUsuari;
  IF intComptador = 0 THEN
    RAISE USUARI_NO_EXISTEIX;
  END IF;  

  IF NOT (strIMEI IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM DISPOSITIU WHERE IMEI = strIMEI;
    IF intComptador > 0 THEN
      RAISE IMEI_EXISTEIX;
    END IF; 
  END IF;  
  
  IF strModelDispositiu IS NULL THEN
    RAISE MODEL_NULL;
  END IF; 

  SELECT COUNT(*) INTO intComptador FROM MODELDISPOSITIU WHERE CODI = strModelDispositiu;
  IF intComptador = 0 THEN
    RAISE MODEL_NO_EXISTEIX;
  END IF;

  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;
  END IF;

  IF strSO IS NULL THEN
    RAISE SO_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM SO WHERE CODI = strSO;
  IF intComptador = 0 THEN
    RAISE SO_NO_EXISTEIX;
  END IF;
  
  -- afegim el nou dispositiu a la plataforma
  INSERT INTO DISPOSITIU (IMEI,MODELDISP,RESOLUCIO,USUARI,SO) VALUES (strIMEI,strModelDispositiu,intResolucio,intCodiUsuari,strSO);
  SELECT MAX(CODI) INTO intCodiDispositiu FROM DISPOSITIU WHERE USUARI = intCodiUsuari; -- recuperem el codi del dispositiu acabat de donar d'alta
  
  strPARAMOUT := 'intCodiDispositiu:' || intCodiDispositiu; 
  -- registrem l'execuci del procediment al log
  registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepions
  EXCEPTION
    WHEN USUARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiUsuari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN USUARI_NO_EXISTEIX THEN
      RSP := 'ERROR: l''usuari amb codi ' || intCodiUsuari || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN IMEI_EXISTEIX THEN
      RSP := 'ERROR: el codi IMEI ' || strIMEI || ' ja existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN MODEL_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strModelDispositiu s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN MODEL_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del model del dispositiu ' || strModelDispositiu || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de la resoluci del dispositiu ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN SO_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strSO s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN SO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del sistema operatiu del dispositiu ' || strSO || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);              
              
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proAltaDispositiuUsuari','intCodiUsuari:' || intCodiUsuari || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proModificaDispositiu                                                                     --
-- DESCRIPCIO:             Modificar les dades d'un dispositiu                                                       --
-- PARAMETRES ENTRADA:     intCodiDispositiu (codi del dispositiu que es vol modificar)                              --
--                         strIMEI (nou numero IMEI del dispositiu)                                                  --
--                         strModelDispositiu (nou codi del model del dispositiu)                                    --
--                         intResolucio (nou codi de la resolucio del dispositiu)                                    --
--                         strSO (nou codi del sistema operatiu del dispositiu)                                      --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: DISPOSITIU_NULL (el parametre intCodiDispositiu es NULL)                                  --
--                         DISPOSITIU_NO_EXISTEIX (el codi del dispositiu no existeix)                               --
--                         IMEI_EXISTEIX (el parametre strIMEI ja existeix)                                          --
--                         MODEL_NO_EXISTEIX (el codi del model del dispositiu no existeix)                          --
--                         RESOLUCIO_NO_EXISTEIX (el codi de la resolucio del dispositiu no existeix)                --
--                         SO_NO_EXISTEIX (el codi del sistema operatiu del dispositiu no existeix)                  --
--                         VALORS_NULL (tots els camps a modificar sn NULL)                                         --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proModificaDispositiu (intCodiDispositiu IN DISPOSITIU.CODI%TYPE,strIMEI IN DISPOSITIU.IMEI%TYPE, strModelDispositiu IN DISPOSITIU.MODELDISP%TYPE,
                            intResolucio IN RESOLUCIO.CODI%TYPE,strSO IN SO.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
strSeparador VARCHAR2 (1 CHAR); -- per afegir el separador dels camps a modificar en la construccio de la sentencia SQL que s'ha d'executar
strSQL VARCHAR2(500 CHAR); -- per construir la sentencia SQL que s'ha d'executar
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DISPOSITIU_NULL EXCEPTION; -- excepcio corresponent al codi del dispositiu amb valor NULL
DISPOSITIU_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del dispositiu amb un valor que no existeix
IMEI_EXISTEIX EXCEPTION; -- excepcio corresponent al codi IMEI del dispositiu amb un valor que ja existeix
MODEL_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del model del dispositiu amb un valor que no existeix
RESOLUCIO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi de la resolucio del dispositiu amb un valor que no existeix
SO_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al codi del sistema operatiu del dispositiu amb un valor que no existeix
VALORS_NULL EXCEPTION; -- excepcio corresponent a que tots els camps de modificacio contenen el valor NULL

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intCodiDispositiu IS NULL THEN
    RAISE DISPOSITIU_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM DISPOSITIU WHERE CODI = intCodiDispositiu;
  IF intComptador = 0 THEN
    RAISE DISPOSITIU_NO_EXISTEIX;
  END IF;  

  IF NOT (strIMEI IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM DISPOSITIU WHERE IMEI = strIMEI;
    IF intComptador > 0 THEN
      RAISE IMEI_EXISTEIX;
    END IF; 
  END IF;  
  
  IF NOT (strModelDispositiu IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM MODELDISPOSITIU WHERE CODI = strModelDispositiu;
    IF intComptador = 0 THEN
      RAISE MODEL_NO_EXISTEIX;
    END IF;
  END IF;

  IF NOT (intResolucio IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM RESOLUCIO WHERE CODI = intResolucio;
    IF intComptador = 0 THEN
      RAISE RESOLUCIO_NO_EXISTEIX;
    END IF;
  END IF;
  
  IF NOT (strSO IS NULL) THEN
    SELECT COUNT(*) INTO intComptador FROM SO WHERE CODI = strSO;
    IF intComptador = 0 THEN
      RAISE SO_NO_EXISTEIX;
    END IF;
  END IF;
  
  IF (strIMEI IS NULL) AND (strModelDispositiu IS NULL) AND (intResolucio IS NULL) AND (strSO IS NULL) THEN
    RAISE VALORS_NULL;
  END IF;
  
  -- Construim la sentencia SQL d'actualitzaci que s'ha d'executar tenint en compte els parmetres informats
  strSeparador := '';
  strSQL := 'UPDATE DISPOSITIU SET ';
  
  IF NOT (strIMEI IS NULL) THEN
    strSQL := strSQL || ' IMEI = ''' || strIMEI || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strModelDispositiu IS NULL) THEN
    strSQL := strSQL || strSeparador || ' MODELDISP = ''' || strModelDispositiu || '''';
    strSeparador := ',';
  END IF;

  IF NOT (intResolucio IS NULL) THEN
    strSQL := strSQL || strSeparador || ' RESOLUCIO = ''' || intResolucio || '''';
    strSeparador := ',';
  END IF;

  IF NOT (strSO IS NULL) THEN
    strSQL := strSQL || strSeparador || ' SO = ''' || strSO || '''';
  END IF;
  
  strSQL := strSQL || ' WHERE CODI = ' || intCodiDispositiu;
  -- executem la sentncia SQL acabada de formar
  EXECUTE IMMEDIATE strSQL;
  -- registrem l'execuci del procediment al log
  registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DISPOSITIU_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intCodiDispositiu s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN DISPOSITIU_NO_EXISTEIX THEN
      RSP := 'ERROR: el dispositiu amb codi ' || intCodiDispositiu || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
              
    WHEN IMEI_EXISTEIX THEN
      RSP := 'ERROR: el codi IMEI ' || strIMEI || ' ja existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN MODEL_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del model del dispositiu ' || strModelDispositiu || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
  
    WHEN RESOLUCIO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi de la resoluci del dispositiu ' || intResolucio || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN SO_NO_EXISTEIX THEN
      RSP := 'ERROR: el codi del sistema operatiu del dispositiu ' || strSO || ' no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);

    WHEN VALORS_NULL THEN
      RSP := 'ERROR: tots els camps sn NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
              
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proModificaDispositiu','intCodiDispositiu:' || intCodiDispositiu || ', strIMEI:' || strIMEI 
              || ', strModelDispositiu:' || strModelDispositiu || ',intResolucio:' || intResolucio
              || ',strSO:' || strSO,strPARAMOUT,RSP,strRTN);
END;
/

------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proDescarrega                                                                              --
-- DESCRIPCIO:             Realitzar la descrrega d'un fitxer binari per part d'un usuari (mitjanant un dispositiu) --
-- PARAMETRES ENTRADA:     intDispositiu (id del dispositiu des del qual es fa la descarrega)                         --
--                         intBinari (id del binari que es descarrega)                                                --
--                         intModePagament (id del mode de pagament que ha utilitzat l'usuari)                        --
-- PARAMETRES SORTIDA:     RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)  --
-- EXCEPCIONS CONTROLADES: DISPOSITIU_NULL (el parametre intDispositiu es NULL)                                       --
--                         DISPOSITIU_NO_EXISTEIX (el parametre intDispositiu no existeix)                            --
--                         BINARI_NULL (el parametre intBinari es NULL)                                               --
--                         BINARI_NO_EXISTEIX (el parametre intBinari no existeix)                                    --
--                         PAGAMENT_NULL (el parametre intModePagament es NULL)                                       --
--                         PAGAMENT_NO_EXISTEIX (el parametre intModePagament no existeix)                            --
------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proDescarrega (intDispositiu IN DISPOSITIU.CODI%TYPE,intBinari IN BINARI.CODI%TYPE, intModePagament IN PAGAMENT.CODI%TYPE,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intCodiAplicacio NUMBER; -- per a recollir el codi de l'aplicacio corresponent al binari
intCodiUsuari NUMBER; -- per a recollir el codi de l'usuari propietari del dispositiu
strOperador CHAR(4 CHAR); -- per a recollir el codi de l'operador de l'usuari propietari del dispositiu
strPais CHAR(2 CHAR); -- per a recollir el codi del pais de l'usuari propietari del dispositiu
intPreu NUMBER(4,2); -- per a recollir l'import de la descarrega
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
DISPOSITIU_NULL EXCEPTION; -- excepcio corresponent al dispositiu d'entrada amb valor NULL
DISPOSITIU_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al dispositiu d'entrada amb un valor que no existeix
BINARI_NULL EXCEPTION; -- excepcio corresponent al binari d'entrada amb valor NULL
BINARI_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al binari d'entrada amb un valor que no existeix
PAGAMENT_NULL EXCEPTION; -- excepcio corresponent al mode de pagament d'entrada amb valor NULL
PAGAMENT_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al mode de pagament d'entrada amb un valor que no existeix

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF intDispositiu IS NULL THEN
    RAISE DISPOSITIU_NULL;
  END IF;

  IF intBinari IS NULL THEN
    RAISE BINARI_NULL;
  END IF;

  IF intModePagament IS NULL THEN
    RAISE PAGAMENT_NULL;
  END IF;
  
  SELECT COUNT(*) INTO intComptador FROM DISPOSITIU WHERE CODI = intDispositiu;
  IF intComptador = 0 THEN
    RAISE DISPOSITIU_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM BINARI WHERE CODI = intBinari;
  IF intComptador = 0 THEN
    RAISE BINARI_NO_EXISTEIX;
  END IF;  

  SELECT COUNT(*) INTO intComptador FROM PAGAMENT WHERE CODI = intModePagament;
  IF intComptador = 0 THEN
    RAISE PAGAMENT_NO_EXISTEIX;
  END IF;  

  -- Cerquem les dades que falten
  SELECT CODIAPLICACIO INTO intCodiAplicacio FROM BINARI WHERE CODI=intBinari;
  SELECT USUARI INTO intCodiUsuari FROM DISPOSITIU WHERE CODI=intDispositiu;
  SELECT OPERADOR,PAIS INTO strOperador,strPais FROM USUARI WHERE CODI=intCodiUsuari;
  SELECT PREU INTO intPreu FROM APLICACIOPREU WHERE CODIAPLICACIO=intCodiAplicacio AND CODIPAIS=strPais; -- Es presuposa que existira el preu ja que sin no s'hauria d'haver pogut fer la descarrega
  
  -- afegim la nova descrrega a la plataforma
  INSERT INTO DESCARREGA (CODIDISPOSITIU,CODIBINARI,CODIUSUARI,CODIAPLICACIO,MODEPAGAMENT,OPERADOR,PREU,PAIS) 
  VALUES (intDispositiu,intBinari,intCodiUsuari,intCodiAplicacio,intModePagament,strOperador,intPreu,strPais);
  -- registrem l'execuci del procediment al log
  registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN DISPOSITIU_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intDispositiu s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN DISPOSITIU_NO_EXISTEIX THEN
      RSP := 'ERROR: el parmetre d''entrada intDispositiu no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intBinari s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN BINARI_NO_EXISTEIX THEN
      RSP := 'ERROR: el parmetre d''entrada intBinari no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN PAGAMENT_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada intModePagament s NULL';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN PAGAMENT_NO_EXISTEIX THEN
      RSP := 'ERROR: el parmetre d''entrada intModePagament no existeix';
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      ROLLBACK; -- descartem les operacions per a no deixar la base de dades incoherent
      registreLOG('proDescarrega','intDispositiu:' || intDispositiu || ', intBinari:' || intBinari || ', intModePagament:' || intModePagament,strPARAMOUT,RSP,strRTN);
END;
/

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatDesPais                                                                                                                               --
-- DESCRIPCIO:             Obtenir un llistat de tots els desenvolupadors d'un pais donat, totes les dades i el nombre d'aplicacions diferents publicades                  --
-- PARAMETRES ENTRADA:     pais (codi ISO 3166-1 alfa-2 del pais del qual es volen consultar els desenvolupadors)                                                          --
-- PARAMETRES SORTIDA:     tblLlistatDesenvolupadors (taula de tipus tblTAULALLISDESPAIS amb el resultat de la consulta)                                                   --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                                                       --
-- EXCEPCIONS CONTROLADES: PAIS_NULL (el pais passat com a parametre d'entrada es NULL)                                                                                    --
--                         PAIS_NO_EXISTEIX (el pais passat com a parametre d'entrada no es valid, no existeix a la taula de paisos)                                       --
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatDesPais (strPais IN PAIS.CODI%TYPE,tblLlistatDesenvolupadors OUT tblTAULALLISDESPAIS,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regLLISDESPAIS typREGISTRELLISDESPAIS := typREGISTRELLISDESPAIS(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorDesenvolupadors IS -- cursor amb els desenvolupadors del pais passat com a entrada
  SELECT CODI,IDDESENVOLUPADOR,DATAALTA,DATABAIXA,NOMEMPRESA,NOMREPLEGAL,ADREA,TELEFON FROM DESENVOLUPADOR WHERE PAIS = strPais ORDER BY IDDESENVOLUPADOR; 
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
PAIS_NULL EXCEPTION; -- excepcio corresponent al pais d'entrada amb valor NULL
PAIS_NO_EXISTEIX EXCEPTION; -- excepcio corresponent al pais d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strPais IS NULL THEN
    RAISE PAIS_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM PAIS WHERE CODI = strPais;
  IF intComptador = 0 THEN
    RAISE PAIS_NO_EXISTEIX;
  END IF;
  
  tblLlistatDesenvolupadors := tblTAULALLISDESPAIS(); -- inicialitzem el parmetre de sortida
  strPARAMOUT := '';

  -- es recorren tots els desenvolupadors del cursor buscant per cadascun d'ells les aplicacions desenvolupades formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDesenvolupadors;
    LOOP 
      FETCH cursorDesenvolupadors
        INTO regLLISDESPAIS.codi,regLLISDESPAIS.idDesenvolupador,regLLISDESPAIS.dataAlta,regLLISDESPAIS.dataBaixa,regLLISDESPAIS.nomEmpresa,regLLISDESPAIS.nomRepLegal,regLLISDESPAIS.adrea,regLLISDESPAIS.telefon;
        EXIT WHEN cursorDesenvolupadors%NOTFOUND;
        
        SELECT COUNT(DISTINCT CODIAPLICACIO) INTO regLLISDESPAIS.appPublicades FROM DESENVOLUPADORBINARI INNER JOIN BINARI ON CODIBINARI = BINARI.CODI WHERE CODIDESENVOLUPADOR=regLLISDESPAIS.codi;
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'codi:' || regLLISDESPAIS.codi || ',' 
                        || 'idDesenvolupador:' || regLLISDESPAIS.idDesenvolupador || ',' 
                        || 'dataAlta:' || regLLISDESPAIS.dataAlta || ',' 
                        || 'dataBaixa:' || regLLISDESPAIS.dataBaixa || ',' 
                        || 'nomEmpresa:' || regLLISDESPAIS.nomEmpresa || ',' 
                        || 'nomRepLegal:' || regLLISDESPAIS.nomRepLegal || ',' 
                        || 'adrea:' || regLLISDESPAIS.adrea || ',' 
                        || 'telefon:' || regLLISDESPAIS.telefon || ',' 
                        || 'appPublicades:' || regLLISDESPAIS.appPublicades;
        tblLlistatDesenvolupadors.extend; -- afegim un nou registre a l'objecte tblLlistatDesenvolupadors
        tblLlistatDesenvolupadors(tblLlistatDesenvolupadors.count) := regLLISDESPAIS; -- informem el nou registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorDesenvolupadors;
  -- registrem l'execuci del procediment al log
  registreLOG('proLlistatDesPais','strPais:' || strPais,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN PAIS_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada PAIS s NULL';
      registreLOG('proLlistatDesPais','strPais:',strPARAMOUT,RSP,strRTN);

    WHEN PAIS_NO_EXISTEIX THEN
      RSP := 'ERROR: el PAIS no existeix';
      registreLOG('proLlistatDesPais','strPais:' || strPais,strPARAMOUT,RSP,strRTN);      

    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proLlistatDesPais','strPais:' || strPais,strPARAMOUT,RSP,strRTN);      
END;
/

-------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatAppActives                                                                                          --
-- DESCRIPCIO:             Obtenir un llistat de totes les aplicacions actives i ordenat pel nmero total de descrregues fins el moment --
-- PARAMETRES ENTRADA:     Cap                                                                                                           --
-- PARAMETRES SORTIDA:     tblLlistatAppActives (taula de tipus tblTAULALLISAPPACTIVES amb el resultat de la consulta)                   --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                     --
-- EXCEPCIONS CONTROLADES: Cap                                                                                                           --
-------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatAppActives (tblLlistatAppActives OUT tblTAULALLISAPPACTIVES,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intI NUMBER; -- per a realitzar la ordenaci dels registres
intJ NUMBER; -- per a realitzar la ordenaci dels registres
intMinim NUMBER; -- per a realitzar la ordenaci dels registres
regLLISAPPACTIVES typREGISTRELLISAPPACTIVES := typREGISTRELLISAPPACTIVES(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorAplicacions IS -- cursor amb les aplicacions actives
  SELECT APLICACIO.CODI,APLICACIO.IDAPLICACIO,APLICACIO.DATAPUJADA,APLICACIO.DATABAIXA,APLICACIO.VERSIO,APLICACIO.LINKVIDEO,RESOLUCIO.NOM 
  FROM APLICACIO LEFT JOIN RESOLUCIO ON APLICACIO.RESOLUCIOMINIMA = RESOLUCIO.CODI WHERE ACTIVA = 'S'; 
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  tblLlistatAppActives := tblTAULALLISAPPACTIVES();
  strPARAMOUT := '';

  -- es recorren totes les aplicacions buscant per cadascuna d'elles les descarregues realitzades formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorAplicacions;
    LOOP 
      FETCH cursorAplicacions
   
        INTO regLLISAPPACTIVES.codi,regLLISAPPACTIVES.idAplicacio,regLLISAPPACTIVES.dataPujada,regLLISAPPACTIVES.dataBaixa,regLLISAPPACTIVES.versio,regLLISAPPACTIVES.linkVideo,regLLISAPPACTIVES.resolucioMinima;
        EXIT WHEN cursorAplicacions%NOTFOUND;
        
        SELECT COUNT(*) INTO regLLISAPPACTIVES.descarregues FROM DESCARREGA WHERE CODIAPLICACIO=regLLISAPPACTIVES.codi;
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'codi:' || regLLISAPPACTIVES.codi || ','
                        || 'idAplicacio:' || regLLISAPPACTIVES.idAplicacio || ','
                        || 'dataPujada:' || regLLISAPPACTIVES.dataPujada || ','
                        || 'dataBaixa:' || regLLISAPPACTIVES.dataBaixa || ','
                        || 'versio:' || regLLISAPPACTIVES.versio || ','
                        || 'linkVideo:' || regLLISAPPACTIVES.linkVideo || ','
                        || 'resolucioMinima:' || regLLISAPPACTIVES.resolucioMinima || ','
                        || 'descarregues:' || regLLISAPPACTIVES.descarregues;
        tblLlistatAppActives.extend; -- afegim un nou registre a l'objecte tblLlistatAppActives
        tblLlistatAppActives(tblLlistatAppActives.count) := regLLISAPPACTIVES; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorAplicacions;

  -- Ordenem els registres utilitzant l'algoritme d'ordenaci per selecci (n2). No s el ms eficient, el quicksort ho s ms (n log n) per si s ms simple i entenedor.
  -- Atenent el nombre d'elements que pot tenir no es considera determinant el cost
  FOR intI IN tblLlistatAppActives.first .. tblLlistatAppActives.last-1 LOOP
    intMinim := intI;
    FOR intJ IN intI+1 .. tblLlistatAppActives.last LOOP
      IF tblLlistatAppActives(intMinim).descarregues > tblLlistatAppActives(intJ).descarregues THEN
        intMinim := intJ;
      END IF;
    END LOOP;
    regLLISAPPACTIVES := tblLlistatAppActives(intI);
    tblLlistatAppActives(intI) := tblLlistatAppActives(intMinim);
    tblLlistatAppActives(intMinim) := regLLISAPPACTIVES ;
  END LOOP;
  -- registrem l'execuci del procediment al log
  registreLOG('proLlistatAppActives','',strPARAMOUT,RSP,strRTN);
  
  -- gesti de l'excepci
  EXCEPTION
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proLlistatAppActives','',strPARAMOUT,RSP,strRTN);      
END;
/

------------------------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatAppAny                                                                                                               --
-- DESCRIPCIO:             Obtenir un llistat de tots els paisos on s'ha descarregat una aplicacio determinada i en un any en concret,                    --
--                         per cada pais el nombre de descarregues de l'aplicacio                                                                         --
-- PARAMETRES ENTRADA:     strCodiApp (codi intern de l'aplicacio APLICACIO(CODI)), si es informat preval sobre el seguent parametre d'entrada strDescApp --
--                         strDescApp (descripci de l'aplicaci APLICACIO(IDAPLICACIO), si strCodiApp es informat, aquest parametre es obviat            --
--                         strAny (any que es vol consultar)                                                                                              --
-- PARAMETRES SORTIDA:     tblLlistatAplicacioAny (taula de tipus tblTAULALLISAPPANY amb el resultat de la consulta)                                      --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                                      --
-- EXCEPCIONS CONTROLADES: APP_NULL (els parametres strCodiApp i strDescApp, ambds, son NULL)                                                            --
--                         APP_NO_EXISTEIX (l'aplicacio no existeix)                                                                                      --
--                         ANY_NULL (el parametre d'entrada strAny es NULL)                                                                               --
--                         ANY_INCORRECTE (el parametre d'entrada strAny no es correcte, no te quatre digits)                                             --
------------------------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatAppAny (strCodiApp IN APLICACIO.CODI%TYPE,strDescApp IN APLICACIO.IDAPLICACIO%TYPE,strAny CHAR,tblLlistatAplicacioAny OUT tblTAULALLISAPPANY,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
intApp NUMBER; -- per a recollir el codi d'una aplicacio
regLLISAPPANY typREGISTRELLISAPPANY := typREGISTRELLISAPPANY(NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorPaisosDesc IS -- cursor amb les descarregues per pais per a l'aplicacio i any d'entrada
  SELECT PAIS.NOM AS pais, COUNT(DESCARREGA.PAIS) AS descarregues
  FROM DESCARREGA LEFT JOIN PAIS ON (DESCARREGA.PAIS = PAIS.CODI) 
  WHERE CODIAPLICACIO = intApp AND TO_CHAR(DATAHORA,'YYYY')=strAny 
  GROUP BY PAIS.NOM 
  ORDER BY PAIS.NOM;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
APP_NULL EXCEPTION; -- excepcio corresponent a l'aplicacio d'entrada amb valor NULL
APP_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a l'aplicacio d'entrada amb un valor inexistent
ANY_NULL EXCEPTION; -- excepcio corresponent a l'any d'entrada amb valor NULL
ANY_INCORRECTE EXCEPTION; -- excepcio corresponent a l'any d'entrada amb un valor que no respon al format 'YYYY'

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF (strCodiApp IS NULL) AND (strDescApp IS NULL) THEN
    RAISE APP_NULL;
  END IF;

  IF strCodiApp IS NOT NULL THEN
    SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE CODI = strCodiApp;
    IF intComptador = 0 THEN
      RAISE APP_NO_EXISTEIX;
    END IF;  
    intApp := strCodiApp;
  ELSE
      SELECT COUNT(*) INTO intComptador FROM APLICACIO WHERE IDAPLICACIO = strDescApp;
      IF intComptador=0 THEN
        RAISE APP_NO_EXISTEIX;
      ELSE
        -- ens guardem el codi de l'aplicacio perque sera util mes endavant
        SELECT CODI INTO intApp FROM APLICACIO WHERE IDAPLICACIO = strDescApp;
      END IF;    
  END IF;

  IF strAny IS NULL THEN
    RAISE ANY_NULL;
  END IF;

  IF LENGTH(strAny) <> 4 THEN
    RAISE ANY_INCORRECTE;
  END IF;

  -- Per a fer la comprobaci de que l'any conte nomes digits, provem a convertir l'string en numero, 
  -- si es produeix l'error INVALID_NUMBER  ORA-01722 amb codi -1722  o 
  -- l'error VALUE_ERROR ORA-06502 amb codi -6502 (consultable amb la funcio SQLCODE) es que no era correcte
  -- la comparaci amb el valor 0 s arbitraria, noms interessa cridar la funcio TO_NUMBER per veure si provoca una excepci
  IF TO_NUMBER(strAny) = 0 THEN
    RAISE ANY_INCORRECTE;
  END IF;
  
  tblLlistatAplicacioAny := tblTAULALLISAPPANY(); -- inicialitzem el parametre de sortida com a buit
  strPARAMOUT := '';

  -- es recorren totes les descarregues per pais formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorPaisosDesc;
    LOOP 
      FETCH cursorPaisosDesc
        INTO regLLISAPPANY.pais,regLLISAPPANY.descarregues;
        EXIT WHEN cursorPaisosDesc%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'pais:' || regLLISAPPANY.pais || ',' 
                        || 'descarregues:' || regLLISAPPANY.descarregues;
        tblLlistatAplicacioAny.extend; -- afegim un nou registre a l'objecte tblLlistatAplicacioAny
        tblLlistatAplicacioAny(tblLlistatAplicacioAny.count) := regLLISAPPANY; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorPaisosDesc;
  -- registrem l'execuci del procediment al log
  registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:' || strAny,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN APP_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strCodiApp i strDescApp sn NULL';
      registreLOG('proLlistatAppAny','strCodiApp:, strDescApp:, strAny:' || strAny,strPARAMOUT,RSP,strRTN);

    WHEN APP_NO_EXISTEIX THEN
      RSP := 'ERROR: l''aplicaci no existeix';
      registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:' || strAny,strPARAMOUT,RSP,strRTN);
      
    WHEN ANY_NULL THEN
      RSP := 'ERROR: l''any s NULL';
      registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:',strPARAMOUT,RSP,strRTN);

    WHEN ANY_INCORRECTE THEN
      RSP := 'ERROR: l''any no t el format correcte: ''YYYY''';
      registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:' || strAny,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      IF (SQLCODE = -1722) OR (SQLCODE = -6502) THEN
        -- Capturem l'error de la funci TO_NUMBER
        RSP := 'ERROR: l''any no t el format correcte: ''YYYY''';
        registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:' || strAny,strPARAMOUT,RSP,strRTN);
      ELSE
        RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
        registreLOG('proLlistatAppAny','strCodiApp:' || strCodiApp || ', strDescApp:' || strDescApp || ', strAny:' || strAny,strPARAMOUT,RSP,strRTN);
      END IF;
END;
/

-----------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatDescUsuari                                                                      --
-- DESCRIPCIO:             Obtenir un llistat de totes les descarregues realitzades per un usuari (numero de mobil)  --
-- PARAMETRES ENTRADA:     strMobil (numero de mobil de l'usuari del qual es volen les descarregues)                 --
-- PARAMETRES SORTIDA:     tblLlistatDescUsuari (taula de tipus tblTAULALLISDESCUSU amb el resultat de la consulta)  --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat) --
-- EXCEPCIONS CONTROLADES: USU_NULL (el parametre strMobil es NULL)                                                  --
--                         USU_NO_EXISTEIX (l'usuari no existeix)                                                    --
-----------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatDescUsuari (strMobil IN USUARI.MOBIL%TYPE,tblLlistatDescUsuari OUT tblTAULALLISDESCUSU,RSP OUT NOCOPY VARCHAR2) IS 

intComptador NUMBER; -- per a recollir valors numerics
regLLISDESCUSU typREGISTRELLISDESCUSU := typREGISTRELLISDESCUSU(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
intUsuari NUMBER; -- per emmagatzemar el codi d'usuari
CURSOR cursorDescUsuari IS -- cursor amb les descarregues de l'usuari
  SELECT DISPOSITIU.IMEI,MODELDISPOSITIU.NOM,APLICACIO.IDAPLICACIO,DESCARREGA.DATAHORA,PAGAMENT.NOM,OPERADOR.NOM,DESCARREGA.PREU,PAIS.NOM
  FROM DESCARREGA LEFT JOIN DISPOSITIU ON (DESCARREGA.CODIDISPOSITIU = DISPOSITIU.CODI) 
    LEFT JOIN MODELDISPOSITIU ON (DISPOSITIU.MODELDISP = MODELDISPOSITIU.CODI)
    LEFT JOIN APLICACIO ON (DESCARREGA.CODIAPLICACIO = APLICACIO.CODI)
    LEFT JOIN PAGAMENT ON (DESCARREGA.MODEPAGAMENT = PAGAMENT.CODI)
    LEFT JOIN OPERADOR ON (DESCARREGA.OPERADOR = OPERADOR.CODI)
    LEFT JOIN PAIS ON (DESCARREGA.PAIS = PAIS.CODI)
  WHERE CODIUSUARI = intUsuari
  ORDER BY DESCARREGA.DATAHORA;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
USU_NULL EXCEPTION; -- excepcio corresponent a l'usuari d'entrada amb valor NULL
USU_NO_EXISTEIX EXCEPTION; -- excepcio corresponent a l'usuari d'entrada amb un valor inexistent

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strMobil IS NULL THEN
    RAISE USU_NULL;
  END IF;

  SELECT COUNT(*) INTO intComptador FROM USUARI WHERE MOBIL = strMobil;
  IF intComptador = 0 THEN
    RAISE USU_NO_EXISTEIX;
  ELSE
    SELECT CODI INTO intUsuari FROM USUARI WHERE MOBIL = strMobil;
  END IF;  
  
  tblLlistatDescUsuari := tblTAULALLISDESCUSU(); -- inicialitzem el parametre de sortida com a buit
  strPARAMOUT := '';

  -- es recorren totes les descarregues de l'usuari formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDescUsuari;
    LOOP 
      FETCH cursorDescUsuari
        INTO regLLISDESCUSU.dispositiuIMEI,regLLISDESCUSU.dispositiuModel,regLLISDESCUSU.aplicacio,
              regLLISDESCUSU.dataHora,regLLISDESCUSU.modePagament,regLLISDESCUSU.operador,regLLISDESCUSU.preu,regLLISDESCUSU.pais;
        EXIT WHEN cursorDescUsuari%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'dispositiuIMEI:' || regLLISDESCUSU.dispositiuIMEI || ',' 
                        || 'dispositiuModel:' || regLLISDESCUSU.dispositiuModel || ',' 
                        || 'aplicacio:' || regLLISDESCUSU.aplicacio || ',' 
                        || 'dataHora:' || regLLISDESCUSU.dataHora || ',' 
                        || 'modePagament:' || regLLISDESCUSU.modePagament || ',' 
                        || 'operador:' || regLLISDESCUSU.operador || ',' 
                        || 'preu:' || regLLISDESCUSU.preu || ',' 
                        || 'pais:' || regLLISDESCUSU.pais;
        tblLlistatDescUsuari.extend; -- afegim un nou registre a l'objecte tblLlistatDescUsuari
        tblLlistatDescUsuari(tblLlistatDescUsuari.count) := regLLISDESCUSU; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorDescUsuari;
  -- registrem l'execuci del procediment al log
  registreLOG('proLlistatDescUsuari','strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN USU_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strMobil s NULL';
      registreLOG('proLlistatDescUsuari','strMobil:',strPARAMOUT,RSP,strRTN);

    WHEN USU_NO_EXISTEIX THEN
      RSP := 'ERROR: l''usuari no existeix';
      registreLOG('proLlistatDescUsuari','strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);
      
    WHEN OTHERS THEN
      RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
      registreLOG('proLlistatDescUsuari','strMobil:' || strMobil,strPARAMOUT,RSP,strRTN);
END;
/

----------------------------------------------------------------------------------------------------------------------------------------------
-- NOM:                    proLlistatTop20Usuari                                                                                            --
-- DESCRIPCIO:             Obtenir un llistat, per una any donat, dels 20 usuaris que ms han gastat en descrregues ordenat de ms a menys --
-- PARAMETRES ENTRADA:     strAny (any del qual es volen obtenir els usuaris top 20 en descarregues)                                        --
-- PARAMETRES SORTIDA:     tblLlistatTop20Usu (taula de tipus tblTAULALLISTOP20USU amb el resultat de la consulta)                          --
--                         RSP (resultat de l'execuci, 'OK' si tot correcte o 'ERROR: TIPUS ERROR' si ha fracassat)                        --
-- EXCEPCIONS CONTROLADES: ANY_NULL (el parametre strAny es NULL)                                                                           --
--                         ANY_INCORRECTE (el parametre d'entrada strAny no es correcte, no te quatre digits)                               --
----------------------------------------------------------------------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE proLlistatTop20Usuari (strAny CHAR,tblLlistatTop20Usu OUT tblTAULALLISTOP20USU,RSP OUT NOCOPY VARCHAR2) IS 

regLLISTOP20USU typREGISTRELLISTOP20USU := typREGISTRELLISTOP20USU(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); -- per a emmagatzemar una fila de la taula resultat
CURSOR cursorDespesesUsuari IS -- cursor amb els vint usuaris que ms han gastat per l'any donat
  SELECT USUARI.CODI,USUARI.DATAALTA,USUARI.DATABAIXA,USUARI.MOBIL,OPERADOR.NOM,USUARI.EMAIL,PAIS.NOM,SUM(PREU) AS despesa 
  FROM DESCARREGA 
  LEFT JOIN USUARI ON (DESCARREGA.CODIUSUARI = USUARI.CODI) 
  LEFT JOIN OPERADOR ON (USUARI.OPERADOR = OPERADOR.CODI)
  LEFT JOIN PAIS ON (USUARI.PAIS = PAIS.CODI)
  WHERE TO_CHAR(DESCARREGA.DATAHORA,'YYYY')=strAny AND rownum <= 20
  GROUP BY USUARI.CODI,USUARI.DATAALTA,USUARI.DATABAIXA,USUARI.MOBIL,OPERADOR.NOM,USUARI.EMAIL,PAIS.NOM
  ORDER BY despesa DESC;
strRTN VARCHAR2(2 CHAR); -- per a recollir el resultat de l'execuci del procediment d'inserir al log
strPARAMOUT VARCHAR2(5000 CHAR); -- per a emmagatzemar la sortida que es registrara al log
ANY_NULL EXCEPTION; -- excepcio corresponent a l'any d'entrada amb valor NULL
ANY_INCORRECTE EXCEPTION; -- excepcio corresponent a l'any d'entrada amb valor un valor que no respon al format 'YYYY'

BEGIN
  RSP := 'OK'; -- inicialitzem a OK l'execuci del procediment, si es produeix un error ja s'actualitzar
  
  -- Comprovaci de les situacions excepcionals
  IF strAny IS NULL THEN
    RAISE ANY_NULL;
  END IF;

  IF LENGTH(strAny) <> 4 THEN
    RAISE ANY_INCORRECTE;
  END IF;

  -- Per a fer la comprobaci de que l'any conte nomes digits, provem a convertir l'string en numero, 
  -- si es produeix l'error INVALID_NUMBER  ORA-01722 amb codi -1722  o 
  -- l'error VALUE_ERROR ORA-06502 amb codi -6502 (consultable amb la funcio SQLCODE) es que no era correcte
  -- la comparaci amb el valor 0 s arbitraria, noms interessa cridar la funcio TO_NUMBER per veure si provoca una excepci
  IF TO_NUMBER(strAny) = 0 THEN
    RAISE ANY_INCORRECTE;
  END IF;
  
  tblLlistatTop20Usu := tblTAULALLISTOP20USU(); -- inicialitzem el parametre de sortida com a buit
  strPARAMOUT := '';

  -- es recorren els top 20 d'usuaris formant el parametre de sortida pel procediment de log i pel propi procediment
	OPEN cursorDespesesUsuari;
    LOOP 
      FETCH cursorDespesesUsuari
        INTO regLLISTOP20USU.codi,regLLISTOP20USU.dataAlta,regLLISTOP20USU.dataBaixa,regLLISTOP20USU.mobil,regLLISTOP20USU.operador,regLLISTOP20USU.email,regLLISTOP20USU.pais,regLLISTOP20USU.despesa;
        EXIT WHEN cursorDespesesUsuari%NOTFOUND;
        
        IF strPARAMOUT IS NOT NULL THEN
          strPARAMOUT := strPARAMOUT || '|';
        END IF;
        strPARAMOUT := strPARAMOUT || 'codi:' || regLLISTOP20USU.codi || ',' 
                        || 'dataAlta:' || regLLISTOP20USU.dataAlta || ',' 
                        || 'dataBaixa:' || regLLISTOP20USU.dataBaixa || ','
                        || 'mobil:' || regLLISTOP20USU.mobil || ','
                        || 'operador:' || regLLISTOP20USU.operador || ',' 
                        || 'email:' || regLLISTOP20USU.email || ',' 
                        || 'pais:' || regLLISTOP20USU.pais || ',' 
                        || 'despesa:' || regLLISTOP20USU.despesa;
        tblLlistatTop20Usu.extend; -- afegim un nou registre a l'objecte tblLlistatTop20Usu
        tblLlistatTop20Usu(tblLlistatTop20Usu.count) := regLLISTOP20USU; -- donem valor al registre acabat de crear a la linia anterior
    END LOOP;
	CLOSE cursorDespesesUsuari;
  -- registrem l'execuci del procediment la log
  registreLOG('proLlistatTop20Usuari','strAny:' || strAny,strPARAMOUT,RSP,strRTN);
  
  -- gesti de les excepcions
  EXCEPTION
    WHEN ANY_NULL THEN
      RSP := 'ERROR: el parmetre d''entrada strAny s NULL';
      registreLOG('proLlistatTop20Usuari','strAny:',strPARAMOUT,RSP,strRTN);

    WHEN ANY_INCORRECTE THEN
      RSP := 'ERROR: l''any no t el format correcte: ''YYYY''';
      registreLOG('proLlistatTop20Usuari','strAny:' || strAny,strPARAMOUT,RSP,strRTN);

    WHEN OTHERS THEN
      IF (SQLCODE = -1722) OR (SQLCODE = -6502) THEN
        -- Capturem l'error de la funci TO_NUMBER
        RSP := 'ERROR: l''any no t el format correcte: ''YYYY''';
        registreLOG('proLlistatTop20Usuari','strAny:' || strAny,strPARAMOUT,RSP,strRTN);
      ELSE
        RSP := 'ERROR: ' || SUBSTR (SQLERRM, 1, 300);
        registreLOG('proLlistatTop20Usuari','strAny:' || strAny,strPARAMOUT,RSP,strRTN);
      END IF;
END;
/

